@@ -1,3 +1,10 @@
+#!/bin/bash
+# restart using a Tcl shell \
+    exec sh -c 'for tclshell in tclsh tclsh83 cygtclsh80 ; do \
+            ( echo | $tclshell ) 2> /dev/null && exec $tclshell "`( cygpath -w \"$0\" ) 2> /dev/null || echo $0`" "$@" ; \
+        done ; \
+        echo "ecosadmin.tcl: cannot find Tcl shell" ; exit 1' "$0" "$@"
+
 # {{{  Banner
 
 #===============================================================================
@@ -7,29 +14,38 @@
 #	A package install/uninstall tool.
 #
 #===============================================================================
-#####COPYRIGHTBEGIN####
-#                                                                          
-# -------------------------------------------                              
-# The contents of this file are subject to the Red Hat eCos Public License 
-# Version 1.1 (the "License"); you may not use this file except in         
-# compliance with the License.  You may obtain a copy of the License at    
-# http://www.redhat.com/                                                   
-#                                                                          
-# Software distributed under the License is distributed on an "AS IS"      
-# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-# License for the specific language governing rights and limitations under 
-# the License.                                                             
-#                                                                          
-# The Original Code is eCos - Embedded Configurable Operating System,      
-# released September 30, 1998.                                             
-#                                                                          
-# The Initial Developer of the Original Code is Red Hat.                   
-# Portions created by Red Hat are                                          
-# Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-# All Rights Reserved.                                                     
-# -------------------------------------------                              
-#                                                                          
-#####COPYRIGHTEND####
+#####ECOSGPLCOPYRIGHTBEGIN####
+## -------------------------------------------
+## This file is part of eCos, the Embedded Configurable Operating System.
+## Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+##
+## eCos is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free
+## Software Foundation; either version 2 or (at your option) any later version.
+##
+## eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+## WARRANTY; without even the implied warranty of MERCHANTABILITY or
+## FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+## for more details.
+##
+## You should have received a copy of the GNU General Public License along
+## with eCos; if not, write to the Free Software Foundation, Inc.,
+## 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+##
+## As a special exception, if other files instantiate templates or use macros
+## or inline functions from this file, or you compile this file and link it
+## with other works to produce a work based on this file, this file does not
+## by itself cause the resulting work to be covered by the GNU General Public
+## License. However the source code for this file must still be made available
+## in accordance with section (3) of the GNU General Public License.
+##
+## This exception does not invalidate any other reasons why a work based on
+## this file might be covered by the GNU General Public License.
+##
+## Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+## at http://sources.redhat.com/ecos/ecos-license/
+## -------------------------------------------
+#####ECOSGPLCOPYRIGHTEND####
 #===============================================================================
 ######DESCRIPTIONBEGIN####
 #
@@ -122,6 +138,9 @@ namespace eval ecosadmin {
 	variable fatal_error_handler ecosadmin::cli_fatal_error
 	variable warning_handler     ecosadmin::cli_warning
 	variable report_handler      ecosadmin::cli_report
+
+        # Keep or remove the CVS directories?
+        variable keep_cvs 0
 }
 
 # }}}
@@ -423,17 +442,28 @@ proc ecosadmin::parse_arguments { argv0 argv } {
 				continue
 			}
 		
+			# Check for --srcdir
+			if { [regexp -- {^-?-srcdir=?([ \.\\/:_a-zA-Z0-9-]*)$} $args($i) dummy match1] == 1 } {
+				if { $match1 == "" } {
+					if { $i == $argc } {
+						puts "ecosrelease: missing argument after --srcdir"
+						exit 1
+					} else {
+						set match1 $args([incr i])
+					}
+				}
+				set ecosadmin::component_repository $match1
+				continue
+			}
+	    
 			# An unrecognised argument.
 			fatal_error "invalid argument $args($i)"
 		}
 	} 
 
-	# Under Windows it is desirable to do some checking on any directories that
-	# have been provided. Some cygwin pathnames that a user might supply are
-	# not acceptable to Tcl.
-	if { $ecosadmin::windows_host } {
-		set ecosadmin::component_repository [get_pathname_for_tcl $ecosadmin::component_repository]
-	}
+	# Convert user-specified UNIX-style Cygwin pathnames to Windows Tcl-style as necessary
+	set ecosadmin::component_repository [get_pathname_for_tcl $ecosadmin::component_repository]
+	set ecosadmin::add_package [get_pathname_for_tcl $ecosadmin::add_package]
 }
 
 #
@@ -586,14 +616,15 @@ proc ecosadmin::read_data { } {
 	
 		set pkgdir [file join $ecosadmin::component_repository $ecosadmin::package_data($pkg,dir)]
 		if { ![file exists $pkgdir] || ![file isdir $pkgdir] } {
-			fatal_error "package $pkg at $pkgdir missing"
-		}
+			warning "package $pkg at $pkgdir missing"
+		} else {
 
-		# Each subdirectory should correspond to a release. A utility routine
-		# is available for this.
-		set ecosadmin::package_data($pkg,versions) [locate_subdirs $pkgdir]
-		if { $ecosadmin::package_data($pkg,versions) == "" } {
-			fatal_error "package $pkg has no version directories"
+                        # Each subdirectory should correspond to a release. A utility routine
+                        # is available for this.
+                        set ecosadmin::package_data($pkg,versions) [locate_subdirs $pkgdir]
+                        if { $ecosadmin::package_data($pkg,versions) == "" } {
+                            fatal_error "package $pkg has no version directories"
+                        }
 		}
 
 		# Sort all the versions using a version-aware comparison version
@@ -642,6 +673,12 @@ proc ecosadmin::locate_files { dir { pattern "*"} } {
 	# Start by getting a list of all the files.
 	set filelist [glob -nocomplain -- [file join $dir $pattern]]
 
+        if { $pattern == "*" } {
+                # For "everything", include ".*" files, but excluding .
+                # and .. directories
+                lappend filelist [glob -nocomplain -- [file join $dir ".\[a-zA-Z0-9\]*"]]
+        }
+
 	# Eliminate the pathnames from all of these files
 	set filenames ""
 	foreach file $filelist {
@@ -650,14 +687,6 @@ proc ecosadmin::locate_files { dir { pattern "*"} } {
 		}
 	}
 
-	# Eliminate any obviously spurious entries.
-	foreach file { CVS Makefile.am Makefile.in makefile acinclude.m4 aclocal.m4 configure.in configure } {
-		set index [lsearch -exact $filenames $file]
-		if { $index != -1 } {
-			set filenames [lreplace $filenames $index $index]
-		}
-	}
-
 	# Eliminate any subdirectories.
 	set subdirs ""
 	foreach name $filenames {
@@ -689,10 +718,12 @@ proc ecosadmin::locate_subdirs { dir { pattern "*" }} {
 	}
 
 	# Get rid of the CVS directory, if any
-	set index [lsearch -exact $dirnames "CVS"]
-	if { $index != -1 } {
-		set dirnames [lreplace $dirnames $index $index]
-	}
+        if { $ecosadmin::keep_cvs == 0 } {
+                set index [lsearch -exact $dirnames "CVS"]
+                if { $index != -1 } {
+                        set dirnames [lreplace $dirnames $index $index]
+                }
+        }
 
 	# That should be it.
 	return $dirnames
@@ -761,7 +792,7 @@ proc ecosadmin::is_empty_directory { dir } {
 
 proc ecosadmin::get_pathname_for_tcl { name } {
 
-	if { $ecosadmin::windows_host } {
+	if { ( $ecosadmin::windows_host ) && ( $name != "" ) } {
 
 		# If there is no logical drive letter specified
 		if { [ string match "?:*" $name ] == 0 } {
@@ -1035,7 +1066,7 @@ proc ecosadmin::filter_old_packages { old_packages } {
 
 # ----------------------------------------------------------------------------
 # Process_add_packages. This routine is responsible for installing packages
-# into the eCos repository using the gunzip and tar tools which must be on
+# into the eCos repository using the gzip and tar tools which must be on
 # the path
 #
 
@@ -1057,12 +1088,12 @@ proc ecosadmin::process_add_package { } {
 	if { $ecosadmin::extract_license_arg == 1 } {
 		# extract the license file (if any) from the specified gzipped tar archive
 		file delete $licensefile
-		catch { exec > $ecosadmin::null_device gunzip < $abs_package | tar xf - $licensefile }
+		catch { exec > $ecosadmin::null_device gzip -d < $abs_package | tar xf - $licensefile }
 		return
 	}
 
 	# extract the package data file from the specified gzipped tar archive
-	if { [ catch { exec > $ecosadmin::null_device gunzip < $abs_package | tar xf - $datafile } message ] != 0 } {
+	if { [ catch { exec > $ecosadmin::null_device gzip -d < $abs_package | tar xf - $datafile } message ] != 0 } {
 		fatal_error "extracting $datafile:\n$message"
 	}
 
@@ -1074,7 +1105,7 @@ proc ecosadmin::process_add_package { } {
 	}
 
 	# extract the remaining package contents and generate a list of extracted files
-	if { [ catch { exec gunzip < $abs_package | tar xvf - > $logfile } message ] != 0 } {
+	if { [ catch { exec gzip -d < $abs_package | tar xvf - > $logfile } message ] != 0 } {
 		file delete $logfile
 		fatal_error "extracting files:\n$message"
 	}
@@ -1164,7 +1195,7 @@ proc ecosadmin::accept_license { archivename filename } {
 	}
 
 	# extract the specified license file from the specified gzipped tar archive
-	if { [ catch { exec > $ecosadmin::null_device gunzip < $archivename | tar xf - $filename } message ] != 0 } {
+	if { [ catch { exec > $ecosadmin::null_device gzip -d < $archivename | tar xf - $filename } message ] != 0 } {
 		# no license file
 		return "y"
 	}
