@@ -5,29 +5,38 @@
 //      Driver API for non-kernel configurations
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -60,11 +69,14 @@
 //--------------------------------------------------------------------------
 // Statics
 
-static volatile cyg_int32 isr_disable_counter;  // ISR disable counter
+static volatile cyg_int32 isr_disable_counter = 1;  // ISR disable counter
 
-volatile cyg_int32 dsr_disable_counter asm("cyg_scheduler_sched_lock");  // DSR disable counter
+static CYG_INTERRUPT_STATE isr_disable_state;
 
-static volatile cyg_interrupt *dsr_list;        // List of pending DSRs
+volatile cyg_int32 dsr_disable_counter  // DSR disable counter
+                      CYGBLD_ATTRIB_ASM_ALIAS( cyg_scheduler_sched_lock );
+
+static cyg_interrupt* volatile dsr_list;        // List of pending DSRs
 
 #ifdef CYGIMP_HAL_COMMON_INTERRUPTS_CHAIN
 
@@ -85,7 +97,7 @@ static void post_dsr( cyg_interrupt *intr )
 
     HAL_DISABLE_INTERRUPTS(old_intr);
 
-        if( intr->dsr_count++ == 0 )
+    if( intr->dsr_count++ == 0 )
     {
         intr->next_dsr = dsr_list;
         dsr_list = intr;
@@ -102,7 +114,7 @@ static void call_dsrs(void)
     
     while( dsr_list != NULL )
     {
-        volatile cyg_interrupt *intr;
+        cyg_interrupt *intr;
         cyg_int32 count;
         CYG_INTERRUPT_STATE old_intr;
 
@@ -167,15 +179,19 @@ interrupt_end(
 
 cyg_uint32 chain_isr(cyg_vector_t vector, CYG_ADDRWORD data)
 {
-    CYG_REPORT_FUNCTION();
- 
     cyg_interrupt *p = *(cyg_interrupt **)data;
+    register cyg_uint32 isr_ret = 0;
+    register cyg_uint32 isr_chain_ret = 0;
+
+    CYG_REPORT_FUNCTION();
 
     while( p != NULL )
     {
         if( p->vector == vector )
         {
-            register cyg_uint32 isr_ret = p->isr(vector, p->data);
+            isr_ret = p->isr(vector, p->data);
+
+            isr_chain_ret |= isr_ret;
 
             if( isr_ret & CYG_ISR_CALL_DSR ) post_dsr(p);
 
@@ -185,9 +201,21 @@ cyg_uint32 chain_isr(cyg_vector_t vector, CYG_ADDRWORD data)
         p = p->next;
     }
 
+#ifdef HAL_DEFAULT_ISR
+    if( (isr_chain_ret & (CYG_ISR_CALL_DSR|CYG_ISR_HANDLED)) == 0 )
+    {
+        // If we finished the loop for some reason other than that an
+        // ISR has handled the interrupt, call any default ISR to either
+        // report the spurious interrupt, or do some other HAL level processing
+        // such as GDB interrupt detection etc.
+
+        HAL_DEFAULT_ISR( vector, 0 );
+    }
+#endif    
+
     CYG_REPORT_RETURN();
     
-    return 0;
+    return isr_ret & CYG_ISR_CALL_DSR;
 }
 
 #endif
@@ -198,11 +226,10 @@ cyg_uint32 chain_isr(cyg_vector_t vector, CYG_ADDRWORD data)
 
 externC void cyg_drv_isr_lock()
 {
-    CYG_INTERRUPT_STATE dummy;
-
     CYG_REPORT_FUNCTION();
 
-    HAL_DISABLE_INTERRUPTS(dummy);
+    if( isr_disable_counter == 0 )
+        HAL_DISABLE_INTERRUPTS(isr_disable_state);
 
     CYG_ASSERT( isr_disable_counter >= 0 , "Disable counter negative");
     
@@ -225,7 +252,7 @@ externC void cyg_drv_isr_unlock()
 
     if ( isr_disable_counter == 0 )
     {
-        HAL_ENABLE_INTERRUPTS();
+        HAL_RESTORE_INTERRUPTS(isr_disable_state);
     }
 
     CYG_REPORT_RETURN();
@@ -334,6 +361,8 @@ externC cyg_bool_t cyg_drv_mutex_trylock( cyg_drv_mutex_t *mutex )
     CYG_REPORT_FUNCTION();
 
     if( mutex->lock == 1 ) result = false;
+
+    mutex->lock = 1;
     
     CYG_REPORT_RETURN();
 
@@ -403,7 +432,7 @@ externC void cyg_drv_cond_destroy( cyg_drv_cond_t  *cond )
 // wakeup only happens if there is a thread actually waiting on the CV
 // when the signal is sent.
 
-externC void cyg_drv_cond_wait( cyg_drv_cond_t *cond )
+externC cyg_bool cyg_drv_cond_wait( cyg_drv_cond_t *cond )
 {
     CYG_REPORT_FUNCTION();
 
@@ -427,7 +456,9 @@ externC void cyg_drv_cond_wait( cyg_drv_cond_t *cond )
 
     cyg_drv_dsr_unlock();
     
-    CYG_REPORT_RETURN();    
+    CYG_REPORT_RETURN();
+
+    return true;
 }
 
 //--------------------------------------------------------------------------
@@ -457,7 +488,107 @@ externC void cyg_drv_cond_broadcast( cyg_drv_cond_t *cond )
     
     CYG_REPORT_RETURN();    
 }
+
+//--------------------------------------------------------------------------
+// Spinlock support.
+// Since we can only support a single CPU in this version of the API, we only
+// set and clear the lock variable to keep track of what's happening.
+
+void cyg_drv_spinlock_init(
+    cyg_drv_spinlock_t  *lock,          /* spinlock to initialize            */
+    cyg_bool_t          locked          /* init locked or unlocked           */
+)
+{
+    CYG_REPORT_FUNCTION();
     
+    lock->lock = locked;
+
+    CYG_REPORT_RETURN();    
+}
+
+void cyg_drv_spinlock_destroy( cyg_drv_spinlock_t *lock )
+{
+    CYG_REPORT_FUNCTION();
+    
+    lock->lock = -1;
+
+    CYG_REPORT_RETURN();    
+}
+
+void cyg_drv_spinlock_spin( cyg_drv_spinlock_t *lock )
+{
+    CYG_REPORT_FUNCTION();
+    
+    CYG_ASSERT( lock->lock == 0 , "Trying to lock locked spinlock");
+
+    lock->lock = 1;
+
+    CYG_REPORT_RETURN();
+}
+
+void cyg_drv_spinlock_clear( cyg_drv_spinlock_t *lock )
+{
+    CYG_REPORT_FUNCTION();
+
+    CYG_ASSERT( lock->lock == 1 , "Trying to clear cleared spinlock");
+
+    lock->lock = 0;
+    
+    CYG_REPORT_RETURN();    
+}
+
+cyg_bool_t cyg_drv_spinlock_try( cyg_drv_spinlock_t *lock )
+{
+    cyg_bool_t result = true;
+    
+    CYG_REPORT_FUNCTION();
+
+    if( lock->lock == 1 ) result = false;
+
+    lock->lock = 1;
+    
+    CYG_REPORT_RETURN();
+
+    return result;
+}
+
+cyg_bool_t cyg_drv_spinlock_test( cyg_drv_spinlock_t *lock )
+{
+    cyg_bool_t result = true;
+    
+    CYG_REPORT_FUNCTION();
+
+    if( lock->lock == 1 ) result = false;
+
+    CYG_REPORT_RETURN();
+
+    return result;
+}
+
+void cyg_drv_spinlock_spin_intsave( cyg_drv_spinlock_t *lock,
+                                    cyg_addrword_t *istate )
+{
+    CYG_REPORT_FUNCTION();
+
+    HAL_DISABLE_INTERRUPTS( *istate );
+
+    lock->lock = 1;
+    
+    CYG_REPORT_RETURN();
+}
+    
+
+void cyg_drv_spinlock_clear_intsave( cyg_drv_spinlock_t *lock,
+                                     cyg_addrword_t istate )
+{
+    CYG_REPORT_FUNCTION();
+
+    lock->lock = 0;
+    
+    HAL_RESTORE_INTERRUPTS( istate );
+    
+    CYG_REPORT_RETURN();
+}
 
 //--------------------------------------------------------------------------
 // Create an interrupt object.
@@ -524,36 +655,39 @@ externC void cyg_drv_interrupt_attach( cyg_handle_t interrupt )
 
     CYG_ASSERT( intr->next == NULL , "cyg_interrupt already on a list");
 
-    cyg_uint32 index;
-
-    HAL_TRANSLATE_VECTOR( intr->vector, index );
-
-    if( chain_list[index] == NULL )
     {
-        // First Interrupt on this chain, just assign it and register
-        // the chain_isr with the HAL.
-        
-        chain_list[index] = intr;
+        cyg_uint32 index;
 
-        HAL_INTERRUPT_ATTACH( intr->vector, chain_isr, &chain_list[index], NULL );
-    }
-    else
-    {
-        // There are already interrupts chained, add this one into the
-        // chain in priority order.
-        
-        Cyg_Interrupt **p = &chain_list[index];
+        HAL_TRANSLATE_VECTOR( intr->vector, index );
 
-        while( *p != NULL )
+        if( chain_list[index] == NULL ) 
         {
-            cyg_interrupt *n = *p;
+            // First Interrupt on this chain, just assign it and
+            // register the chain_isr with the HAL.
+        
+            chain_list[index] = intr;
 
-            if( n->priority < intr->priority ) break;
+            HAL_INTERRUPT_ATTACH( intr->vector, chain_isr, 
+                                  &chain_list[index], NULL );
+        } 
+        else
+        {
+            // There are already interrupts chained, add this one into
+            // the chain in priority order.
+        
+            cyg_interrupt **p = &chain_list[index];
+
+            while( *p != NULL )
+            {
+                cyg_interrupt *n = *p;
+                
+                if( n->priority < intr->priority ) break;
             
-            p = &n->next;
+                p = &n->next;
+            }
+            intr->next = *p;
+            *p = intr;
         }
-        next = *p;
-        *p = intr;
     }
     
 #else
@@ -581,30 +715,33 @@ externC void cyg_drv_interrupt_detach( cyg_handle_t interrupt )
 #ifdef CYGIMP_HAL_COMMON_INTERRUPTS_CHAIN
 
     // Remove the interrupt object from the vector chain.
-    
-    cyg_uint32 index;
 
-    HAL_TRANSLATE_VECTOR( vector, index );
+    {    
+        cyg_uint32 index;
+        cyg_interrupt **p;
 
-    cyg_interrupt **p = &chain_list[index];
+        HAL_TRANSLATE_VECTOR( intr->vector, index );
 
-    while( *p != NULL )
-    {
-        cyg_interrupt *n = *p;
+        p = &chain_list[index];
 
-        if( n == intr )
+        while( *p != NULL )
         {
-            *p = intr->next;
-            break;
-        }
+            cyg_interrupt *n = *p;
             
-        p = &n->next;
-    }
+            if( n == intr )
+            {
+                *p = intr->next;
+                break;
+            }
+            
+            p = &n->next;
+        }
 
-    // If this was the last one, detach the vector.
+        // If this was the last one, detach the vector.
     
-    if( chain_list[index] == NULL )
-        HAL_INTERRUPT_DETACH( intr->vector, chain_isr );
+        if( chain_list[index] == NULL )
+            HAL_INTERRUPT_DETACH( intr->vector, chain_isr );
+    }
     
 #else
     
@@ -618,8 +755,30 @@ externC void cyg_drv_interrupt_detach( cyg_handle_t interrupt )
 
 //--------------------------------------------------------------------------
 // Mask delivery of an interrupt at the interrupt controller.
+// (Interrupt safe)
 
 externC void cyg_drv_interrupt_mask( cyg_vector_t vector )
+{
+    CYG_INTERRUPT_STATE old_ints;
+
+    CYG_REPORT_FUNCTION();
+    CYG_REPORT_FUNCARG1("vector=%d", vector);
+
+    CYG_ASSERT( vector >= CYGNUM_HAL_ISR_MIN, "Invalid vector");    
+    CYG_ASSERT( vector <= CYGNUM_HAL_ISR_MAX, "Invalid vector");
+
+    HAL_DISABLE_INTERRUPTS(old_ints);
+    HAL_INTERRUPT_MASK( vector );
+    HAL_RESTORE_INTERRUPTS(old_ints);
+
+    CYG_REPORT_RETURN();
+}
+
+//--------------------------------------------------------------------------
+// Mask delivery of an interrupt at the interrupt controller.
+// (Not interrupt safe)
+
+externC void cyg_drv_interrupt_mask_intunsafe( cyg_vector_t vector )
 {
     CYG_REPORT_FUNCTION();
     CYG_REPORT_FUNCARG1("vector=%d", vector);
@@ -634,10 +793,33 @@ externC void cyg_drv_interrupt_mask( cyg_vector_t vector )
 
 //--------------------------------------------------------------------------
 // Unmask delivery of an interrupt at the interrupt controller.
+// (Interrupt safe)
 
 externC void cyg_drv_interrupt_unmask( cyg_vector_t vector )
 {
+    CYG_INTERRUPT_STATE old_ints;
+    
     CYG_REPORT_FUNCTION();
+    CYG_REPORT_FUNCARG1("vector=%d", vector);
+
+    CYG_ASSERT( vector >= CYGNUM_HAL_ISR_MIN, "Invalid vector");    
+    CYG_ASSERT( vector <= CYGNUM_HAL_ISR_MAX, "Invalid vector");
+    
+    HAL_DISABLE_INTERRUPTS(old_ints);
+    HAL_INTERRUPT_UNMASK( vector );
+    HAL_RESTORE_INTERRUPTS(old_ints);
+
+    CYG_REPORT_RETURN();
+}
+
+//--------------------------------------------------------------------------
+// Unmask delivery of an interrupt at the interrupt controller.
+// (Not interrupt safe)
+
+externC void cyg_drv_interrupt_unmask_intunsafe( cyg_vector_t vector )
+{
+    CYG_REPORT_FUNCTION();
+    CYG_REPORT_FUNCARG1("vector=%d", vector);
 
     CYG_ASSERT( vector >= CYGNUM_HAL_ISR_MIN, "Invalid vector");    
     CYG_ASSERT( vector <= CYGNUM_HAL_ISR_MAX, "Invalid vector");
@@ -700,6 +882,42 @@ externC void cyg_drv_interrupt_level( cyg_vector_t vector, cyg_priority_t level
     CYG_REPORT_RETURN();
 }
 
+// -------------------------------------------------------------------------
+// CPU interrupt routing
+
+externC void cyg_drv_interrupt_set_cpu( cyg_vector_t vector, cyg_cpu_t cpu )
+{
+    CYG_REPORT_FUNCTION();
+    CYG_REPORT_FUNCARG2("vector = %d, cpu = %d", vector, cpu);
+
+    CYG_ASSERT( vector >= CYGNUM_HAL_ISR_MIN, "Invalid vector");    
+    CYG_ASSERT( vector <= CYGNUM_HAL_ISR_MAX, "Invalid vector");
+
+#ifdef CYGPKG_HAL_SMP_SUPPORT    
+    HAL_INTERRUPT_SET_CPU( vector, cpu );
+#endif
+    
+    CYG_REPORT_RETURN();
+}
+
+externC cyg_cpu_t cyg_drv_interrupt_get_cpu( cyg_vector_t vector )
+{
+    cyg_cpu_t cpu = 0;
+    
+    CYG_REPORT_FUNCTION();
+    CYG_REPORT_FUNCARG1("vector = %d", vector);
+
+    CYG_ASSERT( vector >= CYGNUM_HAL_ISR_MIN, "Invalid vector");    
+    CYG_ASSERT( vector <= CYGNUM_HAL_ISR_MAX, "Invalid vector");
+
+#ifdef CYGPKG_HAL_SMP_SUPPORT    
+    HAL_INTERRUPT_GET_CPU( vector, cpu );
+#endif
+    
+    CYG_REPORT_RETURN();
+
+    return cpu;
+}
 
 // -------------------------------------------------------------------------
 // Exception delivery function called from the HAL as a result of a
