@@ -8,34 +8,44 @@
 //      PowerPC 8xx variant CPU definitions
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+// Copyright (C) 2002 Gary Thomas
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
 // Author(s):    jskov
-// Contributors: jskov
+// Contributors: jskov, gthomas
 // Date:         2000-02-04
 // Purpose:      Provide MPC8xx register definitions
 // Description:  Provide MPC8xx register definitions
@@ -50,6 +60,8 @@
 //
 //==========================================================================
 
+#include <cyg/hal/plf_regs.h>
+
 //--------------------------------------------------------------------------
 // Instruction cache control.
 #define CYGARC_REG_IC_CST          560
@@ -146,7 +158,7 @@
 
 #define MI_RPN_RPNMASK  0xfffff000      // real page no mask
 #define MI_RPN_PPRWRW   0x000008f0      // page protection (rw/rw, page valid)
-#define MI_RPN_LPS      0x00000008      // large page size
+#define MI_RPN_LPS      0x0000000C      // large page size
 #define MI_RPN_SH       0x00000004      // shared page (1 = no ASID cmp)
 #define MI_RPN_CI       0x00000002      // cache inhibited
 #define MI_RPN_V        0x00000001      // entry valid
@@ -167,8 +179,16 @@
 #define MD_DRAM1        826             // data MMU RAM read 1
 
 #define MD_RPN_CHANGED  0x00000100      // page changed
+
 #endif // ifdef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
 
+#define CYGARC_REG_MI_CTR         784
+#define CYGARC_REG_MI_CTR_CIDEF   0x20000000
+
+#define CYGARC_REG_MD_CTR         792
+#define CYGARC_REG_MD_CTR_CIDEF   0x20000000
+#define CYGARC_REG_MD_CTR_WTDEF   0x10000000
+
 //--------------------------------------------------------------------------
 // Internal Memory Map.
 #define CYGARC_REG_IMMR            638  // internal memory map base register
@@ -176,7 +196,9 @@
 #define CYGARC_REG_IMMR_PARTNUM    0x0000ff00 // part number mask (ro)
 #define CYGARC_REG_IMMR_MASKNUM    0x000000ff // mask number mask (ro)
 
-#define CYGARC_REG_IMM_BASE        0xff000000 // the internal memory map base
+#ifndef CYGARC_REG_IMM_BASE        // Can be defined by platform
+#define CYGARC_REG_IMM_BASE        0xfa200000 // the internal memory map base
+#endif
 
 // CP Microcode Revision Number
 #define CYGARC_REG_REV_NUM         (CYGARC_REG_IMM_BASE + 0x3cb0)
@@ -310,6 +332,8 @@
 #define CYGARC_REG_IMM_CICR_IRQMASK 0x0000e000     // irq priority mask
 #define CYGARC_REG_IMM_CICR_IRQ_SHIFT 13
 
+// CPM interrupt in-pending register
+#define CYGARC_REG_IMM_CIPR        (CYGARC_REG_IMM_BASE + 0x944)
 // CPM interrupt mask register
 #define CYGARC_REG_IMM_CIMR        (CYGARC_REG_IMM_BASE + 0x948)
 // CPM interrupt in-service register
