@@ -8,29 +8,38 @@
 //      Macros and prototypes for the tracing system
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -333,17 +342,17 @@ Internal Documentation
 The required functions which are used by the tracing macros are
 
     externC void
-    cyg_tracenomsg( char *psz_func, char *psz_file, cyg_uint32 linenum );
+    cyg_tracenomsg( const char *psz_func, const char *psz_file, 
+                    cyg_uint32 linenum );
     
     externC void
-    cyg_tracemsg( cyg_uint32 what, 
-                  char *psz_func, char *psz_file, cyg_uint32 linenum, 
-                  char *psz_msg );
+    cyg_tracemsg( cyg_uint32 what, const char *psz_func, const char *psz_file,
+                  cyg_uint32 linenum, const char *psz_msg );
     
     externC void
     cyg_tracemsg2( cyg_uint32 what, 
-                   char *psz_func, char *psz_file, cyg_uint32 linenum, 
-                   char *psz_msg,
+                   const char *psz_func, const char *psz_file,
+                   cyg_uint32 linenum, const char *psz_msg,
                    CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1 );
     // extended in the obvious way for 4,6,8 arguments
 
@@ -404,7 +413,7 @@ also fits in a byte as number or a bitset.
 // these are executed to deal with tracing - breakpoint?
 
 externC void
-cyg_tracenomsg( char *psz_func, char *psz_file, cyg_uint32 linenum );
+cyg_tracenomsg( const char *psz_func, const char *psz_file, cyg_uint32 linenum );
 
 externC void
 cyg_trace_dump(void);
@@ -437,31 +446,31 @@ enum cyg_trace_what{
 
 externC void
 cyg_tracemsg( cyg_uint32 what, 
-              char *psz_func, char *psz_file, cyg_uint32 linenum, 
-              char *psz_msg );
+              const char *psz_func, const char *psz_file, cyg_uint32 linenum, 
+              const char *psz_msg );
 
 externC void
 cyg_tracemsg2( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum, 
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum, 
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1 );
 externC void
 cyg_tracemsg4( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum, 
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum, 
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3 );
 externC void
 cyg_tracemsg6( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum, 
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum, 
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3,
                CYG_ADDRWORD arg4,  CYG_ADDRWORD arg5 );
 externC void
 cyg_tracemsg8( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum, 
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum, 
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3,
                CYG_ADDRWORD arg4,  CYG_ADDRWORD arg5,
@@ -602,8 +611,8 @@ class Cyg_TraceFunction_Report_
 {
 public:
     int   cond;
-    char *func;
-    char *file;
+    const char *func;
+    const char *file;
     cyg_uint32 lnum;
 #ifdef CYGDBG_INFRA_DEBUG_TRACE_MESSAGE
     char *exitmsg;
@@ -612,7 +621,7 @@ public:
 #endif
 
     Cyg_TraceFunction_Report_(
-        int condition, char *psz_func, char *psz_file,
+        int condition, const char *psz_func, const char *psz_file,
         cyg_uint32 linenum)
     {
         cond = condition;
@@ -632,7 +641,7 @@ public:
     };
 
     Cyg_TraceFunction_Report_(
-        int condition, char *psz_func, char *psz_file, 
+        int condition, const char *psz_func, const char *psz_file, 
         cyg_uint32 linenum, char *psz_exitmsg )
     {
         cond = condition;
