@@ -5,29 +5,38 @@
 //      Memory buffered trace and assert functions
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -101,9 +110,9 @@ struct Cyg_TraceRecord
 {
     cyg_uint32          what;
     cyg_uint32          tid;
-    char                *function;
-    char                *file;
-    char                *message;
+    const char          *function;
+    const char          *file;
+    const char          *message;
     cyg_uint32          line;
     cyg_uint32          narg;
     CYG_ADDRWORD        arg[8];
@@ -312,7 +321,7 @@ static void print_trace_buffer(void)
         Cyg_TraceRecord *rec = &cyg_infra_trace_buffer[i];
         cyg_uint32 old_ints;
 
-        char *psz_msg = rec->message;
+        const char *psz_msg = rec->message;
         
 
         HAL_DISABLE_INTERRUPTS(old_ints);
@@ -374,11 +383,10 @@ static void increment_buffer_pos()
 #elif defined(CYGDBG_INFRA_DEBUG_TRACE_BUFFER_HALT)
         cyg_infra_trace_buffer_enable = false;
 #elif defined(CYGDBG_INFRA_DEBUG_TRACE_BUFFER_PRINT)
-        print_trace_buffer();
         cyg_infra_trace_buffer_pos = 0;
-        cyg_infra_trace_buffer_wrap = false;
-#else        
-#error No trace buffer full mode set        
+        print_trace_buffer();
+#else
+#error No trace buffer full mode set
 #endif
     }
 
@@ -387,7 +395,7 @@ static void increment_buffer_pos()
 // -------------------------------------------------------------------------
 
 externC void
-cyg_tracenomsg( char *psz_func, char *psz_file, cyg_uint32 linenum )
+cyg_tracenomsg( const char *psz_func, const char *psz_file, cyg_uint32 linenum )
 {
     cyg_uint32 old_ints;
 
@@ -415,8 +423,8 @@ cyg_tracenomsg( char *psz_func, char *psz_file, cyg_uint32 linenum )
 
 externC void
 cyg_tracemsg( cyg_uint32 what, 
-              char *psz_func, char *psz_file, cyg_uint32 linenum,
-              char *psz_msg )
+              const char *psz_func, const char *psz_file, cyg_uint32 linenum,
+              const char *psz_msg )
 {
     cyg_uint32 old_ints;
 
@@ -442,8 +450,8 @@ cyg_tracemsg( cyg_uint32 what,
 
 externC void
 cyg_tracemsg2( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum,
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum,
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1 )
 {
     cyg_uint32 old_ints;
@@ -472,8 +480,8 @@ cyg_tracemsg2( cyg_uint32 what,
 
 externC void
 cyg_tracemsg4( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum,
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum,
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3 )
 {
@@ -505,8 +513,8 @@ cyg_tracemsg4( cyg_uint32 what,
 
 externC void
 cyg_tracemsg6( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum,
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum,
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3,
                CYG_ADDRWORD arg4,  CYG_ADDRWORD arg5 )
@@ -541,8 +549,8 @@ cyg_tracemsg6( cyg_uint32 what,
 
 externC void
 cyg_tracemsg8( cyg_uint32 what, 
-               char *psz_func, char *psz_file, cyg_uint32 linenum,
-               char *psz_msg,
+               const char *psz_func, const char *psz_file, cyg_uint32 linenum,
+               const char *psz_msg,
                CYG_ADDRWORD arg0,  CYG_ADDRWORD arg1,
                CYG_ADDRWORD arg2,  CYG_ADDRWORD arg3,
                CYG_ADDRWORD arg4,  CYG_ADDRWORD arg5,
@@ -712,7 +720,7 @@ cyg_trace_dump(void)
 
 externC void
 cyg_assert_fail( const char *psz_func, const char *psz_file,
-                 cyg_uint32 linenum, const char *psz_msg )
+                 cyg_uint32 linenum, const char *psz_msg ) __THROW
 {
     cyg_uint32 old_ints;
 
@@ -746,14 +754,14 @@ cyg_assert_fail( const char *psz_func, const char *psz_file,
 
 extern "C"
 {
-extern unsigned long CYG_LABEL_NAME(stext);
-extern unsigned long CYG_LABEL_NAME(etext);
+extern unsigned long _stext;
+extern unsigned long _etext;
 
-unsigned long stext_addr = (unsigned long)&CYG_LABEL_NAME(stext);
-unsigned long etext_addr = (unsigned long)&CYG_LABEL_NAME(etext);
+unsigned long stext_addr = (unsigned long)&_stext;
+unsigned long etext_addr = (unsigned long)&_etext;
 };
 
-externC cyg_bool cyg_check_data_ptr(void *ptr)
+externC cyg_bool cyg_check_data_ptr(const void *ptr)
 {
     unsigned long p = (unsigned long)ptr;
     
@@ -764,7 +772,7 @@ externC cyg_bool cyg_check_data_ptr(void *ptr)
     return true;
 }
 
-externC cyg_bool cyg_check_func_ptr(void (*ptr)(void))
+externC cyg_bool cyg_check_func_ptr(const void (*ptr)(void))
 {
     unsigned long p = (unsigned long)ptr;
     
