@@ -5,29 +5,39 @@
 //      Clock class implementations
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+// Copyright (C) 2002 Jonathan Larmour
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -78,19 +88,6 @@ Cyg_Counter::Cyg_Counter(
 
     counter = 0;
     increment = incr;
-#if defined(CYGIMP_KERNEL_COUNTERS_SINGLE_LIST)
-
-    alarm_list = NULL;    // Linear list of Alarms
-
-#elif defined(CYGIMP_KERNEL_COUNTERS_MULTI_LIST)
-
-    for(cyg_ucount32 i=0; i < CYGNUM_KERNEL_COUNTERS_MULTI_LIST_SIZE; i++) {
-        alarm_list[i] = NULL;
-    }
-
-#else
-#error "No CYGIMP_KERNEL_COUNTERS_x_LIST config"
-#endif
 
 }
 
@@ -109,7 +106,7 @@ Cyg_Counter::~Cyg_Counter()
 
 #ifdef CYGDBG_USE_ASSERTS
 
-bool Cyg_Counter::check_this( cyg_assert_class_zeal zeal) const
+cyg_bool Cyg_Counter::check_this( cyg_assert_class_zeal zeal) const
 {
     // check that we have a non-NULL pointer first
     if( this == NULL ) return false;
@@ -154,7 +151,7 @@ void Cyg_Counter::tick( cyg_uint32 ticks )
 
         // now check for any expired alarms
 
-        Cyg_Alarm **alarm_list_ptr;     // pointer to list
+        Cyg_Alarm_List *alarm_list_ptr;     // pointer to list
 
 #if defined(CYGIMP_KERNEL_COUNTERS_SINGLE_LIST)
 
@@ -180,17 +177,17 @@ void Cyg_Counter::tick( cyg_uint32 ticks )
 
         // With a sorted alarm list, we can simply pick alarms off the
         // front of the list until we find one that is in the future.
-        
-        while( *alarm_list_ptr != NULL )
-        {
-            Cyg_Alarm *alarm = *alarm_list_ptr;
 
+        while( !alarm_list_ptr->empty() )
+        {
+            Cyg_Alarm *alarm = alarm_list_ptr->get_head();
+        
             CYG_ASSERTCLASS(alarm, "Bad alarm in counter list" );
             
             if( alarm->trigger <= counter )
             {
                 // remove alarm from list
-                *alarm_list_ptr = alarm->next;
+                alarm_list_ptr->rem_head();
 
                 if( alarm->interval != 0 )
                 {
@@ -210,7 +207,8 @@ void Cyg_Counter::tick( cyg_uint32 ticks )
                 // all done, loop
             }
             else break;
-        }
+            
+        } 
 #else
 
         // With an unsorted list, we must scan the whole list for
@@ -220,20 +218,18 @@ void Cyg_Counter::tick( cyg_uint32 ticks )
         // eliminate alarms we put them onto the done_list and at the
         // end we then move it back to where it belongs.
         
-        Cyg_Alarm *done_list = NULL;
+        Cyg_Alarm_List done_list;
 
-        Cyg_Alarm *alarm_list = *alarm_list_ptr;
-        *alarm_list_ptr = NULL;
+        Cyg_Alarm_List alarm_list;
+
+        alarm_list.merge( *alarm_list_ptr );
         
-        while( alarm_list != NULL )
+        while( !alarm_list.empty() )
         {
-            Cyg_Alarm *alarm = alarm_list;
+            Cyg_Alarm *alarm = alarm_list.rem_head();
 
             CYG_ASSERTCLASS(alarm, "Bad alarm in counter list" );
             
-            // remove alarm from list
-            alarm_list = alarm->next;
-
             if( alarm->trigger <= counter )
             {
                 if( alarm->interval != 0 )
@@ -256,27 +252,18 @@ void Cyg_Counter::tick( cyg_uint32 ticks )
             else
             {
                 // add unused alarm to done list.
-                alarm->next = done_list;
-                done_list = alarm;
+                done_list.add_tail(alarm);
             }
         }
 
-        // Transfer any alarms that might have been added to the
-        // alarm list by alarm callbacks to the done list. This
-        // happens very rarely.
-        while( *alarm_list_ptr != NULL )
-        {
-            Cyg_Alarm *alarm = *alarm_list_ptr;            
-            *alarm_list_ptr = alarm->next;
-            alarm->next = done_list;
-            done_list = alarm;
-        }
+        // Return done list to real list. If any alarms have been
+        // added to the alarm list while we have been scanning then
+        // the done list will be added behind them.
         
-        // return done list to real list
-        *alarm_list_ptr = done_list;
+        alarm_list_ptr->merge( done_list );
         
 #endif        
-        Cyg_Scheduler::unlock();    
+        Cyg_Scheduler::unlock();
 
     }
     
@@ -334,25 +321,24 @@ void Cyg_Counter::add_alarm( Cyg_Alarm *alarm )
     
     CYG_INSTRUMENT_ALARM( ADD, this, alarm );
  
-    {
-        // Find the pointer to the relevant list _after_ a retrigger
-        // alarm has been given its new trigger time.
+    // Find the pointer to the relevant list _after_ a retrigger
+    // alarm has been given its new trigger time.
 
-        Cyg_Alarm **alarm_list_ptr;     // pointer to list
+    Cyg_Alarm_List *alarm_list_ptr;     // pointer to list
 
 #if defined(CYGIMP_KERNEL_COUNTERS_SINGLE_LIST)
 
-        alarm_list_ptr = &alarm_list;
+    alarm_list_ptr = &alarm_list;
 
 #elif defined(CYGIMP_KERNEL_COUNTERS_MULTI_LIST)
 
-        // Each alarm must go into the list that covers the tick that is
-        // going to happen _after_ the trigger time (or at it if trigger
-        // happens to fall on a tick.
+    // Each alarm must go into the list that covers the tick that is
+    // going to happen _after_ the trigger time (or at it if trigger
+    // happens to fall on a tick.
     
-        alarm_list_ptr = &(alarm_list[
-            ((alarm->trigger+increment-1)/increment) %
-                                CYGNUM_KERNEL_COUNTERS_MULTI_LIST_SIZE ] );
+    alarm_list_ptr = &(alarm_list[
+        ((alarm->trigger+increment-1)/increment) %
+        CYGNUM_KERNEL_COUNTERS_MULTI_LIST_SIZE ] );
     
 #else
 #error "No CYGIMP_KERNEL_COUNTERS_x_LIST config"
@@ -360,30 +346,43 @@ void Cyg_Counter::add_alarm( Cyg_Alarm *alarm )
 
 #ifdef CYGIMP_KERNEL_COUNTERS_SORT_LIST
         
-        // Now that we have the list pointer, we can use common code for
-        // both list oragnizations.
+    // Now that we have the list pointer, we can use common code for
+    // both list organizations.
 
-        while( *alarm_list_ptr != NULL )
+    Cyg_Alarm *list_alarm = alarm_list_ptr->get_head();
+
+    if( list_alarm != NULL )
+    {
+        do
         {
-            Cyg_Alarm *list_alarm = *alarm_list_ptr;
-        
             CYG_ASSERTCLASS(list_alarm, "Bad alarm in counter list" );
 
-            // The alarms are in ascending trigger order. When we
-            // find an alarm that is later than us, we go in front of
-            // it.
+            // The alarms are in ascending trigger order. If we
+            // find an alarm that triggers later than us, we go
+            // in front of it.
         
-            if( list_alarm->trigger > alarm->trigger ) break;
-            else alarm_list_ptr = &list_alarm->next;
-        }
-#endif
-        // Insert the new alarm at *alarm_list_ptr
+            if( list_alarm->trigger > alarm->trigger )
+            {
+                alarm_list_ptr->insert( list_alarm, alarm );
+                goto add_alarm_unlock_return;
+            }
 
-        alarm->next = *alarm_list_ptr;
-        *alarm_list_ptr = alarm;
-    
-        Cyg_Scheduler::unlock();            
+            list_alarm = list_alarm->get_next();
+            
+        } while( list_alarm != alarm_list_ptr->get_head() );
+        // a lower or equal alarm time was not found, so drop through
+        // so it is added to the list tail
     }
+    alarm_list_ptr->add_tail( alarm );
+
+ add_alarm_unlock_return:
+#else    
+    
+    alarm_list_ptr->add_tail( alarm );
+        
+#endif
+
+    Cyg_Scheduler::unlock();            
 }
 
 // -------------------------------------------------------------------------
@@ -396,7 +395,7 @@ void Cyg_Counter::rem_alarm( Cyg_Alarm *alarm )
     CYG_ASSERTCLASS( this, "Bad counter object" );
     CYG_ASSERTCLASS( alarm, "Bad alarm passed" );
     
-    Cyg_Alarm **alarm_list_ptr;     // pointer to list
+    Cyg_Alarm_List *alarm_list_ptr;     // pointer to list
 
 #if defined(CYGIMP_KERNEL_COUNTERS_SINGLE_LIST)
 
@@ -418,23 +417,10 @@ void Cyg_Counter::rem_alarm( Cyg_Alarm *alarm )
     Cyg_Scheduler::lock();
 
     CYG_INSTRUMENT_ALARM( REM, this, alarm );
+
+    alarm_list_ptr->remove( alarm );
     
-    while( *alarm_list_ptr != NULL )
-    {
-        Cyg_Alarm *list_alarm = *alarm_list_ptr;
-
-        CYG_ASSERTCLASS(list_alarm, "Bad alarm in counter list" );
-
-        if( list_alarm == alarm ) break;
-        else alarm_list_ptr = &list_alarm->next;
-    }
-
-    // If the alarm was found, remove it from the list.
-    if( *alarm_list_ptr != NULL )
-    {
-        *alarm_list_ptr = alarm->next;
-        alarm->enabled = false;
-    }
+    alarm->enabled = false;
 
     Cyg_Scheduler::unlock();            
 }
@@ -465,7 +451,7 @@ Cyg_Clock::~Cyg_Clock()
 
 #ifdef CYGDBG_USE_ASSERTS
 
-bool Cyg_Clock::check_this( cyg_assert_class_zeal zeal) const
+cyg_bool Cyg_Clock::check_this( cyg_assert_class_zeal zeal) const
 {
     // check that we have a non-NULL pointer first
     if( this == NULL ) return false;
@@ -670,10 +656,6 @@ Cyg_Alarm::Cyg_Alarm(
     interval    = 0;
     enabled     = false;
 
-#if defined(CYGIMP_KERNEL_COUNTERS_SINGLE_LIST) || defined(CYGIMP_KERNEL_COUNTERS_MULTI_LIST)
-    next        = NULL;
-#endif
-    
 }
 
 Cyg_Alarm::Cyg_Alarm(){}
@@ -693,7 +675,7 @@ Cyg_Alarm::~Cyg_Alarm()
 
 #ifdef CYGDBG_USE_ASSERTS
 
-bool Cyg_Alarm::check_this( cyg_assert_class_zeal zeal) const
+cyg_bool Cyg_Alarm::check_this( cyg_assert_class_zeal zeal) const
 {
     // check that we have a non-NULL pointer first
     if( this == NULL ) return false;
@@ -785,6 +767,24 @@ void Cyg_Alarm::enable()
     }
 }
 
+// -------------------------------------------------------------------------
+// Get the current time values from the alarm
+
+void Cyg_Alarm::get_times(
+        cyg_tick_count  *t,      // Next trigger time
+        cyg_tick_count  *i       // Current interval
+        )
+{
+    // Lock the scheduler while we do this to avoid
+    // race conditions.
+    Cyg_Scheduler::lock();
+
+    if( t != NULL ) *t = trigger;
+    if( i != NULL ) *i = interval;
+    
+    Cyg_Scheduler::unlock();
+}
+
 //==========================================================================
 // System clock object
 
@@ -880,10 +880,10 @@ void Cyg_RealTimeClock::dsr(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD
 {
 //    CYG_REPORT_FUNCTION();
 
-#if defined(CYGVAR_KERNEL_COUNTERS_CLOCK_DSR_LATENCY)
+#if defined(CYGVAR_KERNEL_COUNTERS_CLOCK_DSR_LATENCY) && defined(HAL_CLOCK_LATENCY)
     if (measure_clock_latency) {
-        cyg_uint32 delta;
-        HAL_CLOCK_READ(&delta);
+        cyg_int32 delta;
+        HAL_CLOCK_READ((cyg_uint32 *)&delta);
         delta -= clock_dsr_start;
         // Note: Ignore a latency of <= 0 when finding min_clock_latency.
         if (delta > 0 ) {
@@ -905,7 +905,7 @@ void Cyg_RealTimeClock::dsr(cyg_vector vector, cyg_ucount32 count, CYG_ADDRWORD
     rtc->tick( count );
 
 #ifdef CYGSEM_KERNEL_SCHED_TIMESLICE
-#if    0 == CYG_SCHED_UNIQUE_PRIORITIES
+#if    0 == CYGINT_KERNEL_SCHEDULER_UNIQUE_PRIORITIES
 
     // If timeslicing is enabled, call the scheduler to
     // handle it. But not if we have unique priorities.
