@@ -5,29 +5,40 @@
 //      C API Implementation
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+// Copyright (C) 2002 Nick Garnett
+// Copyright (C) 2003 Jonathan Larmour
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -58,9 +69,6 @@
 #include <cyg/kernel/intr.hxx>
 #include <cyg/kernel/clock.hxx>
 
-#include <cyg/kernel/memfixed.hxx>
-#include <cyg/kernel/memvar.hxx>
-
 #include <cyg/kernel/sema.hxx>
 #include <cyg/kernel/flag.hxx>
 #include <cyg/kernel/mutex.hxx>
@@ -115,6 +123,18 @@ externC void cyg_scheduler_lock(void)
                 "Scheduler overlocked" );
 }
 
+/* Lock the scheduler, but never more than level=1. */
+externC void cyg_scheduler_safe_lock(void)
+{
+    Cyg_Scheduler::lock();
+    cyg_ucount32 slock = Cyg_Scheduler::get_sched_lock();
+    if (slock > 1)
+        Cyg_Scheduler::unlock();
+    // get_sched_lock() is unsigned, see below "cyg_ucount32 lock"
+    CYG_ASSERT( (0xff000000 & (Cyg_Scheduler::get_sched_lock())) == 0,
+                "Scheduler overlocked" );
+}
+
 /* Unlock the scheduler. */
 externC void cyg_scheduler_unlock(void)
 {
@@ -125,6 +145,13 @@ externC void cyg_scheduler_unlock(void)
         Cyg_Scheduler::unlock();
 }
 
+/* Read the scheduler lock value. */
+externC cyg_ucount32 cyg_scheduler_read_lock(void)
+{
+    cyg_ucount32 slock = Cyg_Scheduler::get_sched_lock();
+    return slock;
+}
+
 /*---------------------------------------------------------------------------*/
 /* Thread operations */
 
@@ -225,6 +252,9 @@ externC void cyg_thread_set_priority(
 #endif
 }
 
+
+/* Get the normal priority, ie without any applied mutex inheritance or
+ * ceiling protocol. */
 externC cyg_priority_t cyg_thread_get_priority(cyg_handle_t thread)
 {
 #ifdef CYGIMP_THREAD_PRIORITY
@@ -234,6 +264,18 @@ externC cyg_priority_t cyg_thread_get_priority(cyg_handle_t thread)
 #endif
 }
 
+
+/* Get the current priority, ie any applied mutex inheritance or
+ * ceiling protocol. */
+externC cyg_priority_t cyg_thread_get_current_priority(cyg_handle_t thread)
+{
+#ifdef CYGIMP_THREAD_PRIORITY
+    return ((Cyg_Thread *)thread)->get_current_priority();
+#else
+    return 0;
+#endif
+}
+
 /* Deadline scheduling control (optional) */
 
 externC void cyg_thread_deadline_wait( 
@@ -261,6 +303,122 @@ externC cyg_uint32 cyg_thread_get_stack_size(cyg_handle_t thread)
     return ((Cyg_Thread *)thread)->get_stack_size();
 }
 
+#ifdef CYGFUN_KERNEL_THREADS_STACK_MEASUREMENT
+externC cyg_uint32 cyg_thread_measure_stack_usage(cyg_handle_t thread)
+{
+    return ((Cyg_Thread *)thread)->measure_stack_usage();
+}
+#endif
+
+/*---------------------------------------------------------------------------*/
+/* Thread enumeration and information                                        */
+
+#ifdef CYGVAR_KERNEL_THREADS_LIST
+
+cyg_bool_t cyg_thread_get_next( cyg_handle_t *current, cyg_uint16 *id )
+{
+    cyg_bool_t result = true;
+
+    // There is a minute but finite chance that the thread could have
+    // exitted since the previous cyg_thread_get_next() call, and we can't
+    // detect the ID mismatch further down. So be quite zealous with checking.
+
+    CYG_CHECK_DATA_PTRC( current );
+    CYG_CHECK_DATA_PTRC( id );
+    if ( *current != 0 )
+        CYG_CHECK_DATA_PTRC( *current );
+
+    Cyg_Scheduler::lock();
+
+    Cyg_Thread *thread = (Cyg_Thread *)*current;
+    CYG_ASSERT_CLASSC( thread );
+    if( *current == 0 )
+    {
+        thread = Cyg_Thread::get_list_head();
+        *current = (cyg_handle_t)thread;
+        *id = thread->get_unique_id();
+    }
+    else if( (thread->get_unique_id() == *id) &&
+             (thread = thread->get_list_next()) != NULL )
+    {
+        CYG_CHECK_DATA_PTRC( thread );
+        CYG_ASSERT_CLASSC( thread );
+        *current = (cyg_handle_t)thread;
+        *id = thread->get_unique_id();
+    }
+    else
+    {
+        *current = 0;
+        *id = 0;
+        result = false;
+    }
+    
+    Cyg_Scheduler::unlock();
+
+    return result;
+}
+
+cyg_handle_t cyg_thread_find( cyg_uint16 id )
+{
+    Cyg_Scheduler::lock();
+
+    Cyg_Thread *thread = Cyg_Thread::get_list_head();
+
+    while( thread != NULL )
+    {
+        if( thread->get_unique_id() == id )
+            break;
+        
+        thread = thread->get_list_next();
+    }
+
+    Cyg_Scheduler::unlock();
+    
+    return (cyg_handle_t)thread;
+}
+
+#endif
+
+cyg_bool_t cyg_thread_get_info( cyg_handle_t threadh,
+                                cyg_uint16 id,
+                                cyg_thread_info *info )
+{
+    cyg_bool_t result = true;
+    Cyg_Thread *thread = (Cyg_Thread *)threadh;
+    CYG_CHECK_DATA_PTRC( thread );
+    if ( NULL != info )
+        CYG_CHECK_DATA_PTRC( info );
+    
+    Cyg_Scheduler::lock();
+    
+    if( thread->get_unique_id() == id && info != NULL )
+    {
+        CYG_ASSERT_CLASSC( thread );
+        info->handle = threadh;
+        info->id = id;
+        info->state = thread->get_state();
+#ifdef CYGVAR_KERNEL_THREADS_NAME
+        info->name = thread->get_name();
+#else
+        info->name = NULL;
+#endif
+        info->set_pri = thread->get_priority();
+        info->cur_pri = thread->get_current_priority();
+        info->stack_base = thread->get_stack_base();
+        info->stack_size = thread->get_stack_size();
+        
+#ifdef CYGFUN_KERNEL_THREADS_STACK_MEASUREMENT
+        info->stack_used = thread->measure_stack_usage();
+#else
+        info->stack_used = 0;
+#endif
+    }
+    else result = false;
+    
+    Cyg_Scheduler::unlock();
+
+    return result;
+}
 
 /*---------------------------------------------------------------------------*/
 /* Per-thread data                                                           */
@@ -269,28 +427,49 @@ externC cyg_uint32 cyg_thread_get_stack_size(cyg_handle_t thread)
 
 externC cyg_ucount32 cyg_thread_new_data_index()
 {
-        return Cyg_Thread::new_data_index();
+    Cyg_Thread::cyg_data_index index = Cyg_Thread::new_data_index();
+    CYG_ASSERT(index >= 0, "failed to allocate data index" );
+    return index;
 }
 
 externC void cyg_thread_free_data_index(cyg_ucount32 index)
 {
-        Cyg_Thread::free_data_index(index);
+    Cyg_Thread::free_data_index(index);
 }
 
 externC CYG_ADDRWORD cyg_thread_get_data(cyg_ucount32 index)
 {
-        return Cyg_Thread::get_data(index);
+    return Cyg_Thread::get_data(index);
 }
 
 externC CYG_ADDRWORD *cyg_thread_get_data_ptr(cyg_ucount32 index)
 {
-        return Cyg_Thread::get_data_ptr(index);
+    return Cyg_Thread::get_data_ptr(index);
 }
 
 externC void cyg_thread_set_data(cyg_ucount32 index, CYG_ADDRWORD 
 data)
 {
-        Cyg_Thread::self()->set_data(index, data);
+    Cyg_Thread::self()->set_data(index, data);
+}
+#endif
+
+/*---------------------------------------------------------------------------*/
+/* Thread destructors                                                        */
+
+#ifdef CYGPKG_KERNEL_THREADS_DESTRUCTORS
+__externC cyg_bool_t
+cyg_thread_add_destructor( cyg_thread_destructor_fn fn,
+                           cyg_addrword_t data )
+{
+        return Cyg_Thread::self()->add_destructor( fn, data );
+}
+
+__externC cyg_bool_t
+cyg_thread_rem_destructor( cyg_thread_destructor_fn fn,
+                           cyg_addrword_t data )
+{
+        return Cyg_Thread::self()->rem_destructor( fn, data );
 }
 #endif
 
@@ -413,11 +592,20 @@ externC void cyg_interrupt_mask(cyg_vector_t vector)
     Cyg_Interrupt::mask_interrupt( (cyg_vector)vector);
 }
 
+externC void cyg_interrupt_mask_intunsafe(cyg_vector_t vector)
+{
+    Cyg_Interrupt::mask_interrupt_intunsafe( (cyg_vector)vector);
+}
+
 externC void cyg_interrupt_unmask(cyg_vector_t vector)
 {
     Cyg_Interrupt::unmask_interrupt( (cyg_vector)vector);
 }
 
+externC void cyg_interrupt_unmask_intunsafe(cyg_vector_t vector)
+{
+    Cyg_Interrupt::unmask_interrupt_intunsafe( (cyg_vector)vector);
+}
 
 externC void cyg_interrupt_acknowledge(cyg_vector_t vector)
 {
@@ -434,6 +622,27 @@ externC void cyg_interrupt_configure(
     Cyg_Interrupt::configure_interrupt( (cyg_vector)vector, level, up );
 }
 
+externC void cyg_interrupt_set_cpu(
+    cyg_vector_t        vector,         /* vector to control                 */
+    cyg_cpu_t           cpu             /* CPU to set                        */
+)
+{
+#ifdef CYGPKG_KERNEL_SMP_SUPPORT    
+    Cyg_Interrupt::set_cpu( vector, cpu );
+#endif    
+}
+
+externC cyg_cpu_t cyg_interrupt_get_cpu(
+    cyg_vector_t        vector          /* vector to control                 */
+)
+{
+#ifdef CYGPKG_KERNEL_SMP_SUPPORT        
+    return Cyg_Interrupt::get_cpu( vector );
+#else
+    return CYG_KERNEL_CPU_THIS();
+#endif    
+    
+}
 
 /*---------------------------------------------------------------------------*/
 /* Counters, Clocks and Alarms                                               */
@@ -478,6 +687,12 @@ externC void cyg_counter_tick(cyg_handle_t counter)
     ((Cyg_Counter *)counter)->tick(); 
 }
 
+/* Advance counter by multiple ticks                                         */
+externC void cyg_counter_multi_tick(cyg_handle_t counter, cyg_tick_count_t ticks)
+{
+    ((Cyg_Counter *)counter)->tick(ticks); 
+}
+
 /* Create a clock object                */
 externC void cyg_clock_create(
     cyg_resolution_t    resolution,     /* Initial resolution                */
@@ -591,6 +806,17 @@ externC void cyg_alarm_initialize(
         (cyg_tick_count)interval);
 }
 
+externC void cyg_alarm_get_times(
+    cyg_handle_t        alarm,
+    cyg_tick_count_t    *trigger,       /* Next trigger time                 */
+    cyg_tick_count_t    *interval       /* Current interval                  */
+)
+{
+    ((Cyg_Alarm *)alarm)->get_times(
+        (cyg_tick_count*)trigger,
+        (cyg_tick_count*)interval);
+}
+
 externC void cyg_alarm_enable( cyg_handle_t alarm )
 {
     ((Cyg_Alarm *)alarm)->enable();
@@ -687,188 +913,6 @@ externC cyg_bool_t cyg_mbox_waiting_to_put(cyg_handle_t mbox)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/* Memory pools                                                          */
-
-/* Create a variable size memory pool */
-externC void cyg_mempool_var_create(
-    void            *base,              /* base of memory to use for pool */
-    cyg_int32       size,               /* size of memory in bytes        */
-    cyg_handle_t    *handle,            /* returned handle of memory pool */
-    cyg_mempool_var *var                /* space to put pool structure in */
-    )
-{
-    CYG_ASSERT_SIZES( cyg_mempool_var, Cyg_Mempool_Variable );
-
-    Cyg_Mempool_Variable *t = new((void *)var) Cyg_Mempool_Variable (
-        (cyg_uint8 *)base,
-        size
-    );
-    t=t;
-
-    CYG_CHECK_DATA_PTR( handle, "Bad handle pointer" );
-    *handle = (cyg_handle_t)var;
-}
-
-/* Delete variable size memory pool */
-externC void cyg_mempool_var_delete(cyg_handle_t varpool)
-{
-    ((Cyg_Mempool_Variable *)varpool)->~Cyg_Mempool_Variable();
-}
-
-/* Allocates a block of length size.  This waits if the memory is not
-   currently available.  */
-externC void *cyg_mempool_var_alloc(cyg_handle_t varpool, cyg_int32 size)
-{
-    return ((Cyg_Mempool_Variable *)varpool)->alloc(size);
-}
-
-#ifdef CYGFUN_KERNEL_THREADS_TIMER
-
-/* Allocates a block of length size.  This waits for up to delay
-   ticks, if the memory is not already available.  NULL is returned if
-   no memory is available. */
-externC void *cyg_mempool_var_timed_alloc(
-    cyg_handle_t     varpool,
-    cyg_int32        size,
-    cyg_tick_count_t abstime)
-{
-    return ((Cyg_Mempool_Variable *)varpool)->alloc(size, abstime);
-}
-
-#endif
-
-/* Allocates a block of length size.  NULL is returned if no memory is
-   available. */
-externC void *cyg_mempool_var_try_alloc(
-    cyg_handle_t varpool,
-    cyg_int32    size)
-{
-    return ((Cyg_Mempool_Variable *)varpool)->try_alloc(size);
-}
-
-/* Frees memory back into variable size pool. */
-externC void cyg_mempool_var_free(cyg_handle_t varpool, void *p)
-{
-    cyg_bool b;
-    b = ((Cyg_Mempool_Variable *)varpool)->free((cyg_uint8 *)p, 0);
-    CYG_ASSERT( b, "Bad free");
-}
-
-
-/* Returns true if there are any threads waiting for memory in the
-   given memory pool. */
-externC cyg_bool_t cyg_mempool_var_waiting(cyg_handle_t varpool)
-{
-    return ((Cyg_Mempool_Variable *)varpool)->waiting();
-}
-
-/* Puts information about a variable memory pool into the structure
-   provided. */
-externC void cyg_mempool_var_get_info(
-    cyg_handle_t varpool,
-    cyg_mempool_info *info)
-{
-    Cyg_Mempool_Variable *v = (Cyg_Mempool_Variable *)varpool;
-    cyg_uint8 *base;
-    CYG_ADDRWORD maxfree;
-    
-    info->totalmem = v->get_totalmem();
-    info->freemem  = v->get_freemem();
-    v->get_arena(base, info->size, maxfree);
-    info->base = (void *)base;
-    info->blocksize  = -1;
-    info->maxfree = maxfree;
-}
-
-
-/* Create a fixed size memory pool */
-externC void cyg_mempool_fix_create(
-    void            *base,              // base of memory to use for pool
-    cyg_int32       size,               // size of memory in byte
-    cyg_int32       blocksize,          // size of allocation in bytes
-    cyg_handle_t    *handle,            // handle of memory pool
-    cyg_mempool_fix *fix                // space to put pool structure in
-    )
-{
-    CYG_ASSERT_SIZES( cyg_mempool_fix, Cyg_Mempool_Fixed );
-
-    Cyg_Mempool_Fixed *t = new((void *)fix) Cyg_Mempool_Fixed (
-        (cyg_uint8 *)base,
-        size,
-        blocksize
-    );
-    t=t;
-
-    CYG_CHECK_DATA_PTR( handle, "Bad handle pointer" );
-    *handle = (cyg_handle_t)fix;
-}
-
-/* Delete fixed size memory pool */
-externC void cyg_mempool_fix_delete(cyg_handle_t fixpool)
-{
-    ((Cyg_Mempool_Fixed *)fixpool)->~Cyg_Mempool_Fixed();
-}
-
-/* Allocates a block.  This waits if the memory is not
-   currently available.  */
-externC void *cyg_mempool_fix_alloc(cyg_handle_t fixpool)
-{
-    return ((Cyg_Mempool_Fixed *)fixpool)->alloc();
-}
-
-#ifdef CYGFUN_KERNEL_THREADS_TIMER
-
-/* Allocates a block.  This waits for up to delay ticks, if the memory
-   is not already available.  NULL is returned if no memory is
-   available. */
-externC void *cyg_mempool_fix_timed_alloc(
-    cyg_handle_t     fixpool,
-    cyg_tick_count_t abstime)
-{
-    return ((Cyg_Mempool_Fixed *)fixpool)->alloc(abstime);
-}
-
-#endif
-
-/* Allocates a block.  NULL is returned if no memory is available. */
-externC void *cyg_mempool_fix_try_alloc(cyg_handle_t fixpool)
-{
-    return ((Cyg_Mempool_Fixed *)fixpool)->try_alloc();
-}
-
-/* Frees memory back into fixed size pool. */
-externC void cyg_mempool_fix_free(cyg_handle_t fixpool, void *p)
-{
-    cyg_bool b;
-    b = ((Cyg_Mempool_Fixed *)fixpool)->free((cyg_uint8 *)p);
-    CYG_ASSERT( b, "Bad free");
-}
-
-/* Returns true if there are any threads waiting for memory in the
-   given memory pool. */
-externC cyg_bool_t cyg_mempool_fix_waiting(cyg_handle_t fixpool)
-{
-    return ((Cyg_Mempool_Fixed *)fixpool)->waiting();
-}
-
-/* Puts information about a variable memory pool into the structure
-   provided. */
-externC void cyg_mempool_fix_get_info(
-    cyg_handle_t fixpool,
-    cyg_mempool_info *info)
-{
-    Cyg_Mempool_Fixed *f = (Cyg_Mempool_Fixed *)fixpool;
-    cyg_uint8 *base;
-    cyg_addrword_t blocksize;
-    
-    info->totalmem = f->get_totalmem();
-    info->freemem  = f->get_freemem();
-    f->get_arena(base, info->size, blocksize);
-    info->base = (void *)base;
-    info->maxfree = info->blocksize = (cyg_int32)blocksize;
-}
-
 /*---------------------------------------------------------------------------*/
 /* Semaphores                                                                */
 
@@ -888,9 +932,9 @@ externC void cyg_semaphore_destroy( cyg_sem_t *sem )
     ((Cyg_Counting_Semaphore *)sem)->~Cyg_Counting_Semaphore();
 }
 
-externC void cyg_semaphore_wait( cyg_sem_t *sem )
+externC cyg_bool_t cyg_semaphore_wait( cyg_sem_t *sem )
 {
-    ((Cyg_Counting_Semaphore *)sem)->wait();
+    return ((Cyg_Counting_Semaphore *)sem)->wait();
 }
 
 #ifdef CYGFUN_KERNEL_THREADS_TIMER
@@ -1037,6 +1081,24 @@ externC void cyg_mutex_release( cyg_mutex_t *mutex )
     ((Cyg_Mutex *)mutex)->release();
 }
 
+#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_CEILING
+externC void cyg_mutex_set_ceiling( 
+    cyg_mutex_t *mutex, 
+    cyg_priority_t priority )
+{
+    ((Cyg_Mutex *)mutex)->set_ceiling(priority);
+}
+#endif
+
+#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DYNAMIC
+externC void cyg_mutex_set_protocol( 
+    cyg_mutex_t *mutex,
+    enum cyg_mutex_protocol protocol )
+{
+    ((Cyg_Mutex *)mutex)->set_protocol((Cyg_Mutex::cyg_protcol)protocol);
+}
+#endif
+
 /*---------------------------------------------------------------------------*/
 /* Condition Variables                                                       */
 
@@ -1057,9 +1119,9 @@ externC void cyg_cond_destroy( cyg_cond_t *cond )
     ((Cyg_Condition_Variable *)cond)->~Cyg_Condition_Variable();
 }
 
-externC void cyg_cond_wait( cyg_cond_t *cond )
+externC cyg_bool_t cyg_cond_wait( cyg_cond_t *cond )
 {
-    ((Cyg_Condition_Variable *)cond)->wait();
+    return ((Cyg_Condition_Variable *)cond)->wait();
 }
 
 externC void cyg_cond_signal( cyg_cond_t *cond )
@@ -1083,6 +1145,62 @@ externC cyg_bool_t cyg_cond_timed_wait(
 
 #endif
 
+/*---------------------------------------------------------------------------*/
+/* Spinlocks                                                                 */
+
+externC void cyg_spinlock_init(
+    cyg_spinlock_t      *lock,          /* spinlock to initialize            */
+    cyg_bool_t          locked          /* init locked or unlocked           */
+)
+{
+    CYG_ASSERT_SIZES( cyg_spinlock_t, Cyg_SpinLock );
+
+    // Create the spinlock in cleared state
+    Cyg_SpinLock *t = new((void *)lock) Cyg_SpinLock();
+
+    // If the lock is to start locked, then lock it now.
+    if( locked )
+        t->spin();
+}
+
+externC void cyg_spinlock_destroy( cyg_spinlock_t *lock )
+{
+    ((Cyg_SpinLock *)lock)->~Cyg_SpinLock();
+}
+
+externC void cyg_spinlock_spin( cyg_spinlock_t *lock )
+{
+    ((Cyg_SpinLock *)lock)->spin();
+}
+
+externC void cyg_spinlock_clear( cyg_spinlock_t *lock )
+{
+    ((Cyg_SpinLock *)lock)->clear();
+}
+
+externC cyg_bool_t cyg_spinlock_try( cyg_spinlock_t *lock )
+{
+    return ((Cyg_SpinLock *)lock)->trylock();
+}
+
+externC cyg_bool_t cyg_spinlock_test( cyg_spinlock_t *lock )
+{
+    return ((Cyg_SpinLock *)lock)->test();
+}
+
+externC void cyg_spinlock_spin_intsave( cyg_spinlock_t *lock,
+                                cyg_addrword_t *istate )
+{
+    ((Cyg_SpinLock *)lock)->spin_intsave((CYG_INTERRUPT_STATE *)istate);
+}
+    
+externC void cyg_spinlock_clear_intsave( cyg_spinlock_t *lock,
+                                 cyg_addrword_t istate )
+{
+    ((Cyg_SpinLock *)lock)->clear_intsave((CYG_INTERRUPT_STATE)istate);
+}
+    
+
 // -------------------------------------------------------------------------
 // Check structure sizes.
 // This class and constructor get run automatically in debug versions
@@ -1121,17 +1239,16 @@ Cyg_Check_Structure_Sizes::Cyg_Check_Structure_Sizes(int x)
     CYG_CHECK_SIZES( cyg_clock, Cyg_Clock );
     CYG_CHECK_SIZES( cyg_alarm, Cyg_Alarm );
     CYG_CHECK_SIZES( cyg_mbox, Cyg_Mbox );
-    CYG_CHECK_SIZES( cyg_mempool_var, Cyg_Mempool_Variable );
-    CYG_CHECK_SIZES( cyg_mempool_fix, Cyg_Mempool_Fixed );
     CYG_CHECK_SIZES( cyg_sem_t, Cyg_Counting_Semaphore );
     CYG_CHECK_SIZES( cyg_flag_t, Cyg_Flag );
     CYG_CHECK_SIZES( cyg_mutex_t, Cyg_Mutex );
     CYG_CHECK_SIZES( cyg_cond_t, Cyg_Condition_Variable );
+    CYG_CHECK_SIZES( cyg_spinlock_t, Cyg_SpinLock );
     
     CYG_ASSERT( !fail, "Size checks failed");
 }
 
-Cyg_Check_Structure_Sizes check_structure_sizes(1);
+static Cyg_Check_Structure_Sizes cyg_kapi_check_structure_sizes(1);
 
 #endif
 
