@@ -5,29 +5,38 @@
 //      Thread class implementations
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -60,6 +69,10 @@
 #include <cyg/kernel/sched.inl>         // scheduler inlines
 #include <cyg/kernel/clock.inl>         // clock inlines
 
+#ifdef CYGDBG_KERNEL_THREADS_STACK_MEASUREMENT_VERBOSE_EXIT
+#include <cyg/infra/diag.h>
+#endif
+
 // =========================================================================
 // Cyg_HardwareThread members
 
@@ -73,8 +86,8 @@ Cyg_HardwareThread::thread_entry( Cyg_Thread *thread )
 {
     CYG_REPORT_FUNCTION();
 
-    Cyg_Scheduler::scheduler.need_reschedule = false; // finished rescheduling
-    Cyg_Scheduler::scheduler.current_thread = thread; // restore current thread pointer
+    Cyg_Scheduler::scheduler.clear_need_reschedule(); // finished rescheduling
+    Cyg_Scheduler::scheduler.set_current_thread(thread); // restore current thread pointer
 
     CYG_INSTRUMENT_THREAD(ENTER,thread,0);
     
@@ -86,11 +99,11 @@ Cyg_HardwareThread::thread_entry( Cyg_Thread *thread )
     
     // Zero the lock
     HAL_REORDER_BARRIER ();            // Prevent the compiler from moving
-    Cyg_Scheduler::sched_lock = 0;     // the assignment into the code above.
+    Cyg_Scheduler::zero_sched_lock();     // the assignment into the code above.
     HAL_REORDER_BARRIER();
 
     // Call entry point in a loop.
-    
+
     for(;;)
     {
         thread->entry_point(thread->entry_data);
@@ -186,9 +199,12 @@ Cyg_Thread::Cyg_Thread(
 {
     CYG_REPORT_FUNCTION();
 
+    CYG_INSTRUMENT_THREAD(CREATE,this,0);
+    
     // Start the thread in suspended state.
     state               = SUSPENDED;
     suspend_count       = 1;
+    wakeup_count        = 0;
 
     // Initialize sleep_reason which is used by kill, release
     sleep_reason        = NONE;
@@ -202,6 +218,11 @@ Cyg_Thread::Cyg_Thread(
     for( int i = 0; i < CYGNUM_KERNEL_THREADS_DATA_MAX; i++ )
         thread_data[i] = 0;
 #endif
+#ifdef CYGSEM_KERNEL_THREADS_DESTRUCTORS_PER_THREAD
+    for (int j=0; j<CYGNUM_KERNEL_THREADS_DESTRUCTORS; j++) {
+        destructors[j].fn = NULL;
+    }
+#endif
 #ifdef CYGVAR_KERNEL_THREADS_NAME
     name = name_arg;
 #endif
@@ -252,7 +273,7 @@ Cyg_Thread::reinitialize()
     new(this) Cyg_Thread( pri,
                           entry_point, entry_data,
                           name_arg,
-                          stack_base, stack_size );
+                          get_stack_base(), get_stack_size() );
     // the constructor re-registers the thread with the scheduler.
 
     CYG_ASSERTCLASS( this, "Thread corrupted by reinitialize");    
@@ -274,6 +295,9 @@ Cyg_Thread::~Cyg_Thread()
     remove_from_list();
 #endif 
     
+    // Zero the unique_id to render this thread inconsistent.
+    unique_id = 0;
+    
     CYG_REPORT_RETURN();
 }
 
@@ -282,7 +306,7 @@ Cyg_Thread::~Cyg_Thread()
 
 #ifdef CYGDBG_USE_ASSERTS
 
-bool
+cyg_bool
 Cyg_Thread::check_this( cyg_assert_class_zeal zeal) const
 {
 //    CYG_REPORT_FUNCTION();
@@ -302,6 +326,9 @@ Cyg_Thread::check_this( cyg_assert_class_zeal zeal) const
         // of the executing thread.
         if( (stack_ptr > (stack_base + stack_size)) ||
             (stack_ptr < stack_base) ) return false;
+#ifdef CYGFUN_KERNEL_THREADS_STACK_LIMIT
+        if( stack_ptr < stack_limit ) return false;
+#endif
     case cyg_trivial:
     case cyg_none:
     default:
@@ -458,11 +485,7 @@ Cyg_Thread::counted_sleep( cyg_tick_count delay )
             current->sleep();               // prepare to sleep
             current->state |= COUNTSLEEP;   // Set the state
 
-            CYG_ASSERT( Cyg_Scheduler::get_sched_lock() == 1,
-                    "Called with non-zero scheduler lock");
-        
-            Cyg_Scheduler::unlock();
-            Cyg_Scheduler::lock();
+            Cyg_Scheduler::reschedule();
     
             // clear the timer; if it actually fired, no worries.
             clear_timer();
@@ -588,7 +611,7 @@ Cyg_Thread::resume()
     CYG_REPORT_FUNCTION();
 
     CYG_INSTRUMENT_THREAD(RESUME,this,Cyg_Scheduler::current_thread);
-            
+ 
     // Prevent preemption
     Cyg_Scheduler::lock();
 
@@ -615,6 +638,7 @@ Cyg_Thread::resume()
 
     // Unlock the scheduler and maybe switch threads
     Cyg_Scheduler::unlock();
+
     CYG_REPORT_RETURN();
 }
 
@@ -713,25 +737,53 @@ Cyg_Thread::release()
 // -------------------------------------------------------------------------
 // Exit thread. This puts the thread into EXITED state.
 
+#ifdef CYGPKG_KERNEL_THREADS_DESTRUCTORS
+#ifndef CYGSEM_KERNEL_THREADS_DESTRUCTORS_PER_THREAD
+Cyg_Thread::Cyg_Destructor_Entry
+Cyg_Thread::destructors[ CYGNUM_KERNEL_THREADS_DESTRUCTORS ];
+#endif
+#endif
+
 void
 Cyg_Thread::exit()
 {
+    CYG_REPORT_FUNCTION();
+    
     // The thread should never return from this function.
 
     Cyg_Thread *self = Cyg_Thread::self();
 
+#ifdef CYGPKG_KERNEL_THREADS_DESTRUCTORS
+    cyg_ucount16 i;
+    for (i=0; i<CYGNUM_KERNEL_THREADS_DESTRUCTORS; i++) {
+        if (NULL != self->destructors[i].fn) {
+            destructor_fn fn = self->destructors[i].fn;
+            CYG_ADDRWORD data = self->destructors[i].data;
+            fn(data);
+        }        
+    }
+#endif
+#ifdef CYGDBG_KERNEL_THREADS_STACK_MEASUREMENT_VERBOSE_EXIT
+    diag_printf( "Stack usage for thread %08x: %d\n", self,
+		 self->measure_stack_usage() );
+#endif
+
     Cyg_Scheduler::lock();
 
     // clear the timer; if there was none, no worries.
     clear_timer();
 
-    self->state = EXITED;
+    // It is possible that we have already been killed by another
+    // thread, in which case we do not want to try and take ourself
+    // out of the scheduler again.
+    if( self->state != EXITED )
+    {
+        self->state = EXITED;
 
-    Cyg_Scheduler::scheduler.rem_thread(self);
+        Cyg_Scheduler::scheduler.rem_thread(self);
+    }
 
-    // Un-nest any scheduler locks we have until we
-    // suspend.
-    for( ;; ) Cyg_Scheduler::unlock();
+    Cyg_Scheduler::reschedule();
 }
 
 // -------------------------------------------------------------------------
@@ -820,7 +872,7 @@ Cyg_Thread::set_priority( cyg_priority new_priority )
 //    CYG_ASSERT( new_priority >=  CYG_THREAD_MAX_PRIORITY, "Priority out of range");
 //    CYG_ASSERT( new_priority <=  CYG_THREAD_MIN_PRIORITY, "Priority out of range");
     
-    CYG_INSTRUMENT_THREAD(PRIORITY,this,priority);
+    CYG_INSTRUMENT_THREAD(PRIORITY,this,new_priority);
         
     // Prevent preemption
     Cyg_Scheduler::lock();
@@ -843,7 +895,7 @@ Cyg_Thread::set_priority( cyg_priority new_priority )
 
     Cyg_Scheduler::scheduler.deregister_thread(this);
     
-#if CYG_SCHED_UNIQUE_PRIORITIES
+#if CYGINT_KERNEL_SCHEDULER_UNIQUE_PRIORITIES
 
     // Check that there are no other threads at this priority.
     // If so, leave is as it is.
@@ -853,9 +905,9 @@ Cyg_Thread::set_priority( cyg_priority new_priority )
     if( Cyg_Scheduler::scheduler.unique(new_priority) )
         priority = new_priority;
 
-#else // !CYG_SCHED_UNIQUE_PRIORITIES
+#else // !CYGINT_KERNEL_SCHEDULER_UNIQUE_PRIORITIES
 
-#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INHERITANCE_SIMPLE
+#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_SIMPLE
 
     // When we have priority inheritance, we must update the original
     // priority and not the inherited one.  If the new priority is
@@ -876,7 +928,7 @@ Cyg_Thread::set_priority( cyg_priority new_priority )
 
 #endif
     
-#endif // CYG_SCHED_UNIQUE_PRIORITIES
+#endif // CYGINT_KERNEL_SCHEDULER_UNIQUE_PRIORITIES
 
     Cyg_Scheduler::scheduler.register_thread(this);
     
@@ -899,9 +951,13 @@ Cyg_Thread::set_priority( cyg_priority new_priority )
     // priority is less than that of some other runnable thread, in
     // practice checking that is as expensive as what the scheduler
     // will do anyway).
+    // If it is not the current thread then we need to see whether
+    // it is more worthy of execution than any current thread and
+    // rescheduled if necessary.
     
     if( this == Cyg_Scheduler::get_current_thread() )
-        Cyg_Scheduler::need_reschedule = true;
+         Cyg_Scheduler::set_need_reschedule();
+    else Cyg_Scheduler::set_need_reschedule(this);
     
     // Unlock the scheduler and maybe switch threads
     Cyg_Scheduler::unlock();
@@ -999,16 +1055,19 @@ Cyg_Thread::deliver_exception(
 
 // Set the data map bits for each free slot in the data array.
 cyg_ucount32 Cyg_Thread::thread_data_map = (~CYGNUM_KERNEL_THREADS_DATA_ALL) &
-                                           ((1<<CYGNUM_KERNEL_THREADS_DATA_MAX)-1);
+             (1+(((cyg_ucount32)(1<<(CYGNUM_KERNEL_THREADS_DATA_MAX-1))-1)<<1));
+// the second expression is equivalent to ((1<<CYGNUM_KERNEL_THREADS_DATA_MAX)-1);
+// but avoids overflow. The compiler will compile to a constant just fine.
 
-cyg_ucount32
+Cyg_Thread::cyg_data_index
 Cyg_Thread::new_data_index()
 {
     Cyg_Scheduler::lock();
 
-    cyg_ucount32 index;
+    Cyg_Thread::cyg_data_index index;
 
-    CYG_ASSERT( thread_data_map != 0 , "No more thread data indexes");
+    if (0 == thread_data_map)
+        return -1;
     
     // find ls set bit
     HAL_LSBIT_INDEX( index, thread_data_map );
@@ -1021,7 +1080,7 @@ Cyg_Thread::new_data_index()
     return index;
 }
 
-void Cyg_Thread::free_data_index( cyg_ucount32 index )
+void Cyg_Thread::free_data_index( Cyg_Thread::cyg_data_index index )
 {
     Cyg_Scheduler::lock();
 
@@ -1033,6 +1092,56 @@ void Cyg_Thread::free_data_index( cyg_ucount32 index )
 
 #endif
 
+// -------------------------------------------------------------------------
+// Allocate some memory at the lower end of the stack
+// by moving the stack limit pointer.
+
+#if defined(CYGFUN_KERNEL_THREADS_STACK_LIMIT) && \
+    defined(CYGFUN_KERNEL_THREADS_STACK_CHECKING)
+// if not doing stack checking, implementation can be found in thread.inl
+// This implementation puts the magic buffer area (to watch for overruns
+// *above* the stack limit, i.e. there is no official demarcation between
+// the stack and the buffer. But that's okay if you think about it... having
+// a demarcation would not accomplish anything more.
+void *Cyg_HardwareThread::increment_stack_limit( cyg_ucount32 size )
+{
+    void *ret = (void *)stack_limit;
+
+    // First lock the scheduler because we're going to be tinkering with
+    // the check data
+    Cyg_Scheduler::lock();
+
+    // if we've inc'd the limit before, it will be off by the check data
+    // size, so lets correct it
+    if (stack_limit != stack_base)
+        stack_limit -= CYGNUM_KERNEL_THREADS_STACK_CHECK_DATA_SIZE;
+    stack_limit += size;
+
+    // determine base of check data by rounding up to nearest word aligned
+    // address if not already aligned
+    cyg_uint32 *p = (cyg_uint32 *)((stack_limit + 3) & ~3);
+    // i.e. + sizeof(cyg_uint32)-1) & ~(sizeof(cyg_uint32)-1);
+    cyg_ucount32 i;
+    cyg_uint32 sig = (cyg_uint32)this;
+    
+    for ( i = 0;
+          i < CYGNUM_KERNEL_THREADS_STACK_CHECK_DATA_SIZE/sizeof(cyg_uint32);
+          i++ ) {
+        p[i] = (sig ^ (i * 0x01010101));
+    }
+
+    // increment limit by the check size. Note this will not necessarily
+    // reach the end of the check data. But that doesn't really matter.
+    // Doing this allows better checking of the saved stack pointer in
+    // Cyg_Thread::check_this()
+    stack_limit += CYGNUM_KERNEL_THREADS_STACK_CHECK_DATA_SIZE;
+
+    Cyg_Scheduler::unlock();
+    
+    return ret;
+}
+#endif
+    
 // =========================================================================
 // Cyg_ThreadTimer member functions
 
@@ -1116,10 +1225,10 @@ Cyg_ThreadTimer::alarm(
 # endif // CYGNUM_KERNEL_THREADS_IDLE_STACK_SIZE
 #endif // CYGNUM_HAL_STACK_SIZE_MINIMUM
 
-static char idle_thread_stack[CYGNUM_KERNEL_THREADS_IDLE_STACK_SIZE];
+static char idle_thread_stack[CYGNUM_KERNEL_CPU_MAX][CYGNUM_KERNEL_THREADS_IDLE_STACK_SIZE];
 
 // Loop counter for debugging/housekeeping
-cyg_uint32 idle_thread_loops = 1;
+cyg_uint32 idle_thread_loops[CYGNUM_KERNEL_CPU_MAX];
 
 // -------------------------------------------------------------------------
 // Idle thread code.
@@ -1131,9 +1240,9 @@ idle_thread_main( CYG_ADDRESS data )
 
     for(;;)
     {
-        idle_thread_loops++;
+        idle_thread_loops[CYG_KERNEL_CPU_THIS()]++;
 
-        HAL_IDLE_THREAD_ACTION(idle_thread_loops);
+        HAL_IDLE_THREAD_ACTION(idle_thread_loops[CYG_KERNEL_CPU_THIS()]);
 
 #if 0
         // For testing, it is useful to be able to fake
@@ -1156,47 +1265,35 @@ idle_thread_main( CYG_ADDRESS data )
 class Cyg_IdleThread : public Cyg_Thread
 {
 public:
-    Cyg_IdleThread(
-        cyg_thread_entry        *entry,           // entry point function
-        CYG_ADDRWORD            entry_data,       // entry data
-        cyg_ucount32            stack_size = 0,   // stack size, 0 = use default
-        CYG_ADDRESS             stack_base = 0    // stack base, NULL = allocate
-        );
+    Cyg_IdleThread();
         
 };
 
+// -------------------------------------------------------------------------
+// Instantiate the idle thread
+
+Cyg_IdleThread idle_thread[CYGNUM_KERNEL_CPU_MAX] CYG_INIT_PRIORITY( IDLE_THREAD );
+
 // -------------------------------------------------------------------------
 // Idle threads constructor
 
-Cyg_IdleThread::Cyg_IdleThread(
-    cyg_thread_entry        *entry,           // entry point function
-    CYG_ADDRWORD            entry_data,       // entry data
-    cyg_ucount32            stack_size,       // stack size, 0 = use default
-    CYG_ADDRESS             stack_base        // stack base, NULL = allocate
-    )
+Cyg_IdleThread::Cyg_IdleThread()
     : Cyg_Thread( CYG_THREAD_MIN_PRIORITY,
-                  entry,
-                  entry_data,
+                  idle_thread_main,
+                  0,
                   "Idle Thread",
-                  stack_base,
-                  stack_size)
+                  (CYG_ADDRESS)idle_thread_stack[this-&idle_thread[0]],
+                  CYGNUM_KERNEL_THREADS_IDLE_STACK_SIZE)
 {
     CYG_REPORT_FUNCTION();
 
-    resume();
+    // Call into scheduler to set up this thread as the default
+    // current thread for its CPU.
+    
+    Cyg_Scheduler::scheduler.set_idle_thread( this, this-&idle_thread[0] );
+
     CYG_REPORT_RETURN();
 }
 
-// -------------------------------------------------------------------------
-// Instantiate the idle thread
-
-Cyg_IdleThread idle_thread CYG_INIT_PRIORITY( IDLE_THREAD ) =
-Cyg_IdleThread( idle_thread_main,
-                0,
-                CYGNUM_KERNEL_THREADS_IDLE_STACK_SIZE,
-                CYG_ADDRESS(idle_thread_stack)
-    ); 
-
-
 // -------------------------------------------------------------------------
 // EOF common/thread.cxx
