@@ -5,29 +5,38 @@
 //        Mutex test 3 - priority inheritance
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -37,6 +46,7 @@
 // Description:   Tests mutex priority inheritance
 //####DESCRIPTIONEND####
 
+#include <pkgconf/hal.h>
 #include <pkgconf/kernel.h>
 
 #include <cyg/kernel/sched.hxx>        // Cyg_Scheduler::start()
@@ -51,6 +61,11 @@
 
 #include <cyg/infra/diag.h>             // diag_printf
 
+#ifdef CYGSEM_HAL_STOP_CONSTRUCTORS_ON_FLAG
+externC void
+cyg_hal_invoke_constructors();
+#endif
+
 // ------------------------------------------------------------------------
 //
 // These checks should be enough; any other scheduler which has priorities
@@ -58,7 +73,55 @@
 // so the test should work correctly.
 
 #if defined(CYGVAR_KERNEL_COUNTERS_CLOCK) &&    \
-    (CYGNUM_KERNEL_SCHED_PRIORITIES > 20)
+    (CYGNUM_KERNEL_SCHED_PRIORITIES > 20) &&    \
+    !defined(CYGPKG_KERNEL_SMP_SUPPORT)
+
+// ------------------------------------------------------------------------
+// Manufacture a simpler feature test macro for priority inheritance than
+// the configuration gives us. We have priority inheritance if it is configured
+// as the only protocol, or if it is the default protocol for dynamic protocol
+// choice.
+// FIXME: If we have dynamic protocol choice, we can also set priority inheritance
+// as the protocol to be used on the mutexes we are interested in. At present we
+// do not do this.
+
+#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_INHERIT
+# ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DYNAMIC
+#  ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_INHERIT
+#   define PRIORITY_INHERITANCE "dynamic-default-inherit"
+#  endif
+# else
+#  define PRIORITY_INHERITANCE "static-inherit"
+# endif
+#endif
+
+#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_CEILING
+# ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DYNAMIC
+#  ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_CEILING
+#   if CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_PRIORITY <= 5
+#    define PRIORITY_INHERITANCE "dynamic-default-ceiling-high"
+#   elif CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_PRIORITY >= 15
+#    define NO_PRIORITY_INHERITANCE "dynamic-default-ceiling-low"
+#   else
+#    define PRIORITY_UNKNOWN "dynamic-default-ceiling-mid"
+#   endif
+#  endif
+# else
+#  if CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_PRIORITY <= 5
+#   define PRIORITY_INHERITANCE "static-ceiling-high"
+#  elif CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INVERSION_PROTOCOL_DEFAULT_PRIORITY >= 15
+#   define NO_PRIORITY_INHERITANCE "static-ceiling-low"
+#  else
+#   define PRIORITY_UNKNOWN "static-ceiling-mid"
+#  endif
+# endif
+#endif
+
+#ifndef PRIORITY_INHERITANCE
+# ifndef NO_PRIORITY_INHERITANCE
+#  define NO_PRIORITY_INHERITANCE "no scheme selected"
+# endif
+#endif
 
 // ------------------------------------------------------------------------
 // Management functions
@@ -84,26 +147,28 @@ static CYG_ALIGNMENT_TYPE stack[NTHREADS] [
    (STACKSIZE+sizeof(CYG_ALIGNMENT_TYPE)-1)
      / sizeof(CYG_ALIGNMENT_TYPE)                     ];
 
-static int nthreads = 0;
+static volatile int nthreads = 0;
 
 static Cyg_Thread *new_thread( cyg_thread_entry *entry,
                                CYG_ADDRWORD data,
                                CYG_ADDRWORD priority,
                                int do_resume )
 {
-    CYG_ASSERT(nthreads < NTHREADS, 
+    int _nthreads = nthreads++;
+
+    CYG_ASSERT(_nthreads < NTHREADS, 
                "Attempt to create more than NTHREADS threads");
 
-    thread[nthreads] = new( (void *)&thread_obj[nthreads] )
+    thread[_nthreads] = new( (void *)&thread_obj[_nthreads] )
         Cyg_Thread(priority,
                    entry, data, 
                    NULL,                // no name
-                   (CYG_ADDRESS)stack[nthreads], STACKSIZE );
+                   (CYG_ADDRESS)stack[_nthreads], STACKSIZE );
 
     if ( do_resume )
-        thread[nthreads]->resume();
+        thread[_nthreads]->resume();
 
-    return thread[nthreads++];
+    return thread[_nthreads];
 }
 
 
@@ -250,14 +315,18 @@ static void t2( CYG_ADDRWORD data )
         // Wait longer than the delay in t3 waiting on go_flag
     } while ( now < (then + 3) );
 
-#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INHERITANCE
-    CYG_TEST_INFO( "Checking for MUTEX_PRIORITY_INHERITANCE" );
+#ifdef PRIORITY_UNKNOWN
+    CYG_TEST_INFO( "Not checking: " PRIORITY_UNKNOWN );
+#else
+#ifdef PRIORITY_INHERITANCE
+    CYG_TEST_INFO( "Checking priority scheme: " PRIORITY_INHERITANCE );
     CYG_TEST_CHECK( 1 == t3ran, "Thread 3 did not run" );
     CYG_TEST_CHECK( 1 == got_it, "Thread 1 did not get the mutex" );
 #else
-    CYG_TEST_INFO( "Checking for NO mutex_priority_inheritance" );
+    CYG_TEST_INFO( "Checking NO priority scheme: " NO_PRIORITY_INHERITANCE );
     CYG_TEST_CHECK( 0 == t3ran, "Thread 3 DID run" );
     CYG_TEST_CHECK( 0 == got_it, "Thread 1 DID get the mutex" );
+#endif
 #endif
 
     CYG_TEST_CHECK( 0 == t3ended, "Thread 3 ended prematurely [T2,1]" );
@@ -336,19 +405,11 @@ static void control_thread( CYG_ADDRWORD data )
         if ( cyg_test_is_simulator && (0 != i && 13 != i && 26 != i) )
             continue;    // 13 is 111 base 3, 26 is 222 base 3
 
-#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INHERITANCE
+#ifdef PRIORITY_INHERITANCE
         // If the simple scheme plus relay enhancement, or any other
         // *complete* scheme, we can run all three ancillary threads no
         // problem, so no special action here.
 
-#ifdef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INHERITANCE_SIMPLE
-#ifndef CYGSEM_KERNEL_SYNCH_MUTEX_PRIORITY_INHERITANCE_SIMPLE_RELAY
-        // If the simple scheme but no relay enhancement:
-        if ( k )                        // Cannot run thread 2a nor 3a
-            break;                      //     if no priority relay.
-#endif
-#endif
-
 #else
         // If no priority inheritance at all, running threads 1a and 2a is
         // OK, but not thread 3a; it blocks the world.
@@ -391,6 +452,9 @@ static void control_thread( CYG_ADDRWORD data )
 externC void
 cyg_user_start( void )
 { 
+#ifdef CYGSEM_HAL_STOP_CONSTRUCTORS_ON_FLAG
+    cyg_hal_invoke_constructors();
+#endif
     new_thread( control_thread, 0, 2, 1 );
 }
 
@@ -402,7 +466,8 @@ cyg_start( void )
     CYG_TEST_INIT();
     CYG_TEST_PASS_FINISH("Mutex3 test requires:\n"
                          "CYGVAR_KERNEL_COUNTERS_CLOCK &&\n"
-                         "(CYGNUM_KERNEL_SCHED_PRIORITIES > 20)\n");
+                         "(CYGNUM_KERNEL_SCHED_PRIORITIES > 20) &&\n"
+                         "!defined(CYGPKG_KERNEL_SMP_SUPPORT)\n");
 }
 #endif // CYGVAR_KERNEL_COUNTERS_CLOCK &c
 
