@@ -5,29 +5,38 @@
 //        Basic thread stress test
 //
 //==========================================================================
-//####COPYRIGHTBEGIN####
-//                                                                          
-// -------------------------------------------                              
-// The contents of this file are subject to the Red Hat eCos Public License 
-// Version 1.1 (the "License"); you may not use this file except in         
-// compliance with the License.  You may obtain a copy of the License at    
-// http://www.redhat.com/                                                   
-//                                                                          
-// Software distributed under the License is distributed on an "AS IS"      
-// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
-// License for the specific language governing rights and limitations under 
-// the License.                                                             
-//                                                                          
-// The Original Code is eCos - Embedded Configurable Operating System,      
-// released September 30, 1998.                                             
-//                                                                          
-// The Initial Developer of the Original Code is Red Hat.                   
-// Portions created by Red Hat are                                          
-// Copyright (C) 1998, 1999, 2000 Red Hat, Inc.                             
-// All Rights Reserved.                                                     
-// -------------------------------------------                              
-//                                                                          
-//####COPYRIGHTEND####
+//####ECOSGPLCOPYRIGHTBEGIN####
+// -------------------------------------------
+// This file is part of eCos, the Embedded Configurable Operating System.
+// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
+//
+// eCos is free software; you can redistribute it and/or modify it under
+// the terms of the GNU General Public License as published by the Free
+// Software Foundation; either version 2 or (at your option) any later version.
+//
+// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or
+// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+// for more details.
+//
+// You should have received a copy of the GNU General Public License along
+// with eCos; if not, write to the Free Software Foundation, Inc.,
+// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
+//
+// As a special exception, if other files instantiate templates or use macros
+// or inline functions from this file, or you compile this file and link it
+// with other works to produce a work based on this file, this file does not
+// by itself cause the resulting work to be covered by the GNU General Public
+// License. However the source code for this file must still be made available
+// in accordance with section (3) of the GNU General Public License.
+//
+// This exception does not invalidate any other reasons why a work based on
+// this file might be covered by the GNU General Public License.
+//
+// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
+// at http://sources.redhat.com/ecos/ecos-license/
+// -------------------------------------------
+//####ECOSGPLCOPYRIGHTEND####
 //==========================================================================
 //#####DESCRIPTIONBEGIN####
 //
@@ -52,16 +61,17 @@
 
 #include <cyg/hal/hal_arch.h>
 
-#if defined(CYGPKG_KERNEL) && defined(CYGPKG_IO) && defined(CYGPKG_LIBC)
+#if defined(CYGPKG_KERNEL) && defined(CYGPKG_IO) && defined(CYGPKG_ISOINFRA)
 
 #include <pkgconf/kernel.h>
-#include <pkgconf/libc.h>
+#include <pkgconf/isoinfra.h>
+#include CYGHWR_MEMORY_LAYOUT_H
 
 #if defined(CYGFUN_KERNEL_API_C)
 
 #include <cyg/kernel/kapi.h>
 
-#ifdef CYGPKG_LIBC_STDIO
+#ifdef CYGINT_ISO_STDIO_FORMATTED_IO
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,7 +84,7 @@
 #include <cyg/kernel/test/stackmon.h>
 
 #if defined(CYGFUN_KERNEL_THREADS_TIMER)
-#if defined(CYGPKG_LIBC_MALLOC)
+#if CYGINT_ISO_MALLOC
 
 /* if TIME_LIMIT is defined, it represents the number of seconds this
    test should last; if it is undefined the test will go forever */
@@ -84,14 +94,22 @@
 // STACK_SIZE is typical +2kB for printf family calls which use big
 // auto variables. Add more for handler which calls perform_stressful_tasks()
 #define STACK_SIZE (2*1024 + CYGNUM_HAL_STACK_SIZE_TYPICAL)
-#define STACK_SIZE_HANDLER (4*1024 + CYGNUM_HAL_STACK_SIZE_TYPICAL)
+#define STACK_SIZE_HANDLER (STACK_SIZE + 30*CYGNUM_HAL_STACK_FRAME_SIZE)
 
 #define N_MAIN 1
 
+// If we have instrumentation enabled, make the execution time in the
+// simulator even shorter that we were going to anyway.
+#ifdef CYGPKG_KERNEL_INSTRUMENT
+#define SIM_DELAY_DIVISOR 100
+#else
+#define SIM_DELAY_DIVISOR 10
+#endif
+
 //-----------------------------------------------------------------------
 // Some targets need to define a smaller number of handlers due to
 // memory restrictions.
-#ifdef CYGPKG_HAL_ARM_AEB
+#if defined(CYGMEM_REGION_ram_SIZE) && (CYGMEM_REGION_ram_SIZE < 0x80000)
 #define MAX_HANDLERS 4
 #define N_LISTENERS 1
 #define N_CLIENTS 1
@@ -99,7 +117,7 @@
 #undef STACK_SIZE
 #undef STACK_SIZE_HANDLER
 #define STACK_SIZE (1024 + CYGNUM_HAL_STACK_SIZE_TYPICAL)
-#define STACK_SIZE_HANDLER (1024 + CYGNUM_HAL_STACK_SIZE_TYPICAL)
+#define STACK_SIZE_HANDLER (STACK_SIZE + 10*CYGNUM_HAL_STACK_FRAME_SIZE)
 #endif
 
 //-----------------------------------------------------------------------
@@ -387,11 +405,20 @@ void main_program(cyg_addrword_t data)
         }
 
 #ifdef DEATH_TIME_LIMIT
-        // Stop test if time. We don't care about resource state here
-        // since we've only run for a limited time anyway.
+        // Stop test if time.
         if (is_dead) {
-            print_statistics(1);
-            CYG_TEST_PASS_FINISH("Kernel thread stress test OK");
+            // Pause clients
+            cyg_mutex_lock(&client_request_lock); {
+                clients_paused = 1;
+            } cyg_mutex_unlock(&client_request_lock);
+
+            // When all handlers have stopped, we can print statistics
+            // knowing that all (handler allocated) resources should have
+            // been freed. That is, we should be able to determine leaks.
+            if (0 == handler_thread_in_use_count) {
+                print_statistics(1);
+                CYG_TEST_PASS_FINISH("Kernel thread stress test OK");
+            }
         }
 #endif /* DEATH_TIME_LIMIT */
 
@@ -474,7 +501,7 @@ void start_handler(void)
 {
     int prio;
     char* name;
-    int handler_slot;
+    int handler_slot = 0;
     int found = 0;
 
     while (!found) {
@@ -647,8 +674,8 @@ void setup_death_alarm(cyg_addrword_t data, cyg_handle_t *deathHp,
             ((1000000000.0*rtc_res.divisor)
              *((double)DEATH_TIME_LIMIT)/((double)rtc_res.dividend));
         if ( cyg_test_is_simulator )
-            tick_delay /= 10;
-#ifdef CYGPKG_HAL_I386_LINUX 
+            tick_delay /= SIM_DELAY_DIVISOR;
+#ifdef CYGPKG_HAL_SYNTH
         // 20 seconds is a long time compared to the run time of other tests.
         // Reduce to 10 seconds, allowing more tests to get run.
         tick_delay /= 2;
@@ -692,7 +719,6 @@ cyg_addrword_t sc_thread_create(
 
 #define MINS_HOUR (60)
 #define MINS_DAY  (60*24)
-externC void *cyg_libc_get_malloc_pool( void );
 
 void print_statistics(int print_full)
 {
@@ -759,16 +785,18 @@ void print_statistics(int print_full)
         }
     } cyg_mutex_unlock(&statistics_print_lock);
 
+#if CYGINT_ISO_MALLINFO
     //--------------------------------
     // System information
     {
-        cyg_mempool_info mem_info;
-
-        cyg_mempool_var_get_info((cyg_handle_t) cyg_libc_get_malloc_pool(), 
-                                 &mem_info);
+        struct mallinfo mem_info;
+       
+        mem_info = mallinfo();
+        
         printf(" Memory system: Total=0x%08x Free=0x%08x Max=0x%08x\n", 
-               mem_info.totalmem, mem_info.freemem, mem_info.maxfree);
+               mem_info.arena, mem_info.fordblks, mem_info.maxfree);
     }
+#endif
 
     // Dump stack status
     printf(" Stack usage:\n");
@@ -797,9 +825,9 @@ void print_statistics(int print_full)
 #endif /* (CYGNUM_KERNEL_SCHED_PRIORITIES >=    */
 /* (N_MAIN+N_CLIENTS+N_LISTENERS+MAX_HANDLERS)) */
 
-#else /* CYGSEM_LIBC_MALLOC */
+#else /* CYGINT_ISO_MALLOC */
 # define N_A_MSG "this test needs malloc"
-#endif /* CYGSEM_LIBC_MALLOC */
+#endif /* CYGINT_ISO_MALLOC */
 
 #else /* CYGFUN_KERNEL_THREADS_TIMER */
 # define N_A_MSG "this test needs kernel threads timer"
@@ -809,16 +837,16 @@ void print_statistics(int print_full)
 # define N_A_MSG "this test needs libm"
 #endif /* CYGPKG_LIBM */
 
-#else /* CYGSEM_LIBC_STDIO */
-# define N_A_MSG "this test needs stdio"
-#endif /* CYGSEM_LIBC_STDIO */
+#else /* CYGINT_ISO_STDIO_FORMATTED_IO */
+# define N_A_MSG "this test needs stdio formatted I/O"
+#endif /* CYGINT_ISO_STDIO_FORMATTED_IO */
 
 #else // def CYGFUN_KERNEL_API_C
 # define N_A_MSG "this test needs Kernel C API"
 #endif
 
-#else // def CYGPKG_KERNEL && CYGPKG_IO && CYGPKG_LIBC
-# define N_A_MSG "this tests needs Kernel, libc and IO"
+#else // def CYGPKG_KERNEL && CYGPKG_IO && CYGPKG_ISOINFRA
+# define N_A_MSG "this tests needs Kernel, isoinfra and IO"
 #endif
 
 #ifdef N_A_MSG
