@@ -10,7 +10,8 @@
 //####COPYRIGHTBEGIN####
 //                                                                          
 // ----------------------------------------------------------------------------
-// Copyright (C) 1999, 2000 Red Hat, Inc.
+// Copyright (C) 2002 Bart Veer
+// Copyright (C) 1999, 2000, 2001 Red Hat, Inc.
 //
 // This file is part of the eCos host tools.
 //
@@ -91,10 +92,12 @@ CdlInterpreterBody::CdlInterpreterBody(Tcl_Interp* tcl_interp_arg)
     loadable            = 0;
     container           = 0;
     node                = 0;
-    filename            = "";
+    context             = "";
     error_fn_ptr        = 0;
     warning_fn_ptr      = 0;
     current_commands    = 0;
+    cdl_result          = false;
+    
     CYGDBG_MEMLEAK_CONSTRUCTOR();
     cdlinterpreterbody_cookie   = CdlInterpreterBody_Magic;
 
@@ -143,10 +146,10 @@ CdlInterpreterBody::make(Tcl_Interp* tcl_interp_arg)
         result = new CdlInterpreterBody(tcl_interp);
         
         std::string version = Cdl::get_library_version();
-        if (0 == Tcl_SetVar(tcl_interp, "cdl_version", const_cast<char*>(version.c_str()), TCL_GLOBAL_ONLY)) {
+        if (0 == Tcl_SetVar(tcl_interp, "cdl_version", CDL_TCL_CONST_CAST(char*,version.c_str()), TCL_GLOBAL_ONLY)) {
             throw std::bad_alloc();
         }
-        if (0 == Tcl_SetVar(tcl_interp, "cdl_interactive", const_cast<char*>(Cdl::is_interactive() ? "1" : "0"),
+        if (0 == Tcl_SetVar(tcl_interp, "cdl_interactive", CDL_TCL_CONST_CAST(char*, (Cdl::is_interactive() ? "1" : "0")),
                             TCL_GLOBAL_ONLY)) {
             throw std::bad_alloc();
         }
@@ -191,7 +194,7 @@ CdlInterpreterBody::create_slave(CdlLoadable loadable_arg, bool safe)
 
     // FIXME: creating a slave that is not safe appears to fail.
 #if 0    
-    Tcl_Interp* slave = Tcl_CreateSlave(interp, const_cast<char*>(slave_name.c_str()), safe);
+    Tcl_Interp* slave = Tcl_CreateSlave(interp, CDL_TCL_CONST_CAST(char*, slave_name.c_str()), safe);
 #else
     Tcl_Interp* slave = Tcl_CreateInterp();
 #endif
@@ -264,10 +267,11 @@ CdlInterpreterBody::~CdlInterpreterBody()
     loadable                    = 0;
     container                   = 0;
     node                        = 0;
-    filename                    = "";
+    context                     = "";
     error_fn_ptr                = 0;
     warning_fn_ptr              = 0;
     current_commands            = 0;
+    cdl_result                  = false;
     
     // Make sure slave interpreters get deleted before the current one
     for (std::vector<CdlInterpreter>::iterator i = slaves.begin(); i != slaves.end(); i++) {
@@ -416,14 +420,14 @@ CdlInterpreterBody::get_node() const
 }
 
 std::string
-CdlInterpreterBody::get_filename() const
+CdlInterpreterBody::get_context() const
 {
-    CYG_REPORT_FUNCNAME("CdlInterpreter::get_filename");
+    CYG_REPORT_FUNCNAME("CdlInterpreter::get_context");
     CYG_REPORT_FUNCARG1XV(this);
     CYG_PRECONDITION_THISC();
 
     CYG_REPORT_RETURN();
-    return filename;
+    return context;
 }
 
 CdlDiagnosticFnPtr
@@ -532,27 +536,27 @@ CdlInterpreterBody::pop_node(CdlNode old_node)
 }
 
 std::string
-CdlInterpreterBody::push_filename(std::string new_filename)
+CdlInterpreterBody::push_context(std::string new_context)
 {
-    CYG_REPORT_FUNCNAME("CdlInterpreter::push_filename");
+    CYG_REPORT_FUNCNAME("CdlInterpreter::push_context");
     CYG_REPORT_FUNCARG1XV(this);
     CYG_PRECONDITION_THISC();
-    CYG_PRECONDITIONC("" != new_filename);
+    CYG_PRECONDITIONC("" != new_context);
 
-    std::string result = filename;
-    filename = new_filename;
+    std::string result = context;
+    context = new_context;
     return result;
 }
 
 void
-CdlInterpreterBody::pop_filename(std::string old_filename)
+CdlInterpreterBody::pop_context(std::string old_context)
 {
-    CYG_REPORT_FUNCNAME("CdlInterpreter::pop_filename");
+    CYG_REPORT_FUNCNAME("CdlInterpreter::pop_context");
     CYG_REPORT_FUNCARG1XV(this);
     CYG_PRECONDITION_THISC();
-    CYG_PRECONDITIONC("" != filename);
+    CYG_PRECONDITIONC("" != context);
 
-    filename = old_filename;
+    context = old_context;
 
     CYG_REPORT_RETURN();
 }
@@ -650,34 +654,48 @@ CdlInterpreterBody::eval(std::string script, std::string& str_result)
 
     int result  = TCL_OK;
     int size    = script.size();
- 
+
+    // Distinguish between results set by the Tcl interpreter and results
+    // set by CDL-related commands running in that interpreter.
+    cdl_result = false;
+    
     if (size < 2048) {
         char buf[2048];
         script.copy(buf, size, 0);
         buf[size] = '\0';
         result = Tcl_Eval(tcl_interp, buf);
     } else {
-        char*   buf = static_cast<char*>(malloc(script.size() + 1));
+        char* buf = static_cast<char*>(malloc(script.size() + 1));
         if (0 == buf) {
-            // Stay within Tcl's error handling mechanism rather than
-            // throw an exception.
-            char *msg = "Out of memory";
-            Tcl_SetResult(tcl_interp, msg, TCL_STATIC);
-            str_result = msg;
-            CYG_REPORT_RETVAL(TCL_ERROR);
-            return TCL_ERROR;
+            this->set_result(CdlParse::construct_diagnostic(this, "internal error", "", "Out of memory"));
+            result = TCL_ERROR;
+        } else {
+            script.copy(buf, size, 0);
+            buf[size] = '\0';
+            result = Tcl_Eval(tcl_interp, buf);
+            free(buf);
         }
-        script.copy(buf, size, 0);
-        buf[size] = '\0';
-        result = Tcl_Eval(tcl_interp, buf);
-        free(buf);
     }
-    str_result = Tcl_GetStringResult(tcl_interp);
+
     // The distinction between TCL_OK and TCL_RETURN is probably not worth
     // worrying about.
     if (TCL_RETURN == result) {
         result = TCL_OK;
     }
+    
+    // If we have an error condition that was raised by the Tcl
+    // interpreter rather than by the library, it needs to be
+    // raised up to the library level. That way the error count
+    // etc. are kept accurate.
+    if ((TCL_OK != result) && !cdl_result) {
+        const char* tcl_result = Tcl_GetStringResult(tcl_interp);
+        if ((0 == tcl_result) || ('\0' == tcl_result[0])) {
+            tcl_result = "Internal error, no additional information available.";
+        }
+        CdlParse::report_error(this, "", tcl_result);
+    }
+    
+    str_result = Tcl_GetStringResult(tcl_interp);
     CYG_REPORT_RETVAL(result);
     return result;
 }
@@ -694,33 +712,46 @@ CdlInterpreterBody::eval_cdl_code(const cdl_tcl_code script, std::string& str_re
 
     int result  = TCL_OK;
     int size    = script.size();
+    // Distinguish between results set by the Tcl interpreter and results
+    // set by CDL-related commands running in that interpreter.
+    cdl_result = false;
+    
     if (size < 2048) {
         char buf[2048];
         script.copy(buf, size, 0);
         buf[size] = '\0';
         result = Tcl_Eval(tcl_interp, buf);
     } else {
-        char*   buf = static_cast<char*>(malloc(script.size() + 1));
+        char* buf = static_cast<char*>(malloc(script.size() + 1));
         if (0 == buf) {
-            // Stay within Tcl's error handling mechanism rather than
-            // throw an exception.
-            char *msg = "Out of memory";
-            Tcl_SetResult(tcl_interp, msg, TCL_STATIC);
-            str_result = msg;
-            CYG_REPORT_RETVAL(TCL_ERROR);
-            return TCL_ERROR;
+            this->set_result(CdlParse::construct_diagnostic(this, "internal error", "", "Out of memory"));
+            result = TCL_ERROR;
+        } else {
+            script.copy(buf, size, 0);
+            buf[size] = '\0';
+            result = Tcl_Eval(tcl_interp, buf);
+            free(buf);
         }
-        script.copy(buf, size, 0);
-        buf[size] = '\0';
-        result = Tcl_Eval(tcl_interp, buf);
-        free(buf);
     }
-    str_result = Tcl_GetStringResult(tcl_interp);
     // The distinction between TCL_OK and TCL_RETURN is probably not worth
     // worrying about.
     if (TCL_RETURN == result) {
         result = TCL_OK;
     }
+    
+    // If we have an error condition that was raised by the Tcl
+    // interpreter rather than by the library, it needs to be
+    // raised up to the library level. That way the error count
+    // etc. are kept accurate.
+    if ((TCL_OK != result) && !cdl_result) {
+        const char* tcl_result = Tcl_GetStringResult(tcl_interp);
+        if ((0 == tcl_result) || ('\0' == tcl_result[0])) {
+            tcl_result = "Internal error, no additional information available.";
+        }
+        CdlParse::report_error(this, "", tcl_result);
+    }
+    
+    str_result = Tcl_GetStringResult(tcl_interp);
     CYG_REPORT_RETVAL(result);
     return result;
 }
@@ -734,17 +765,56 @@ CdlInterpreterBody::eval_file(std::string script, std::string& str_result)
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC("" != script);
 
-    int result = Tcl_EvalFile(tcl_interp, const_cast<char*>(script.c_str()));
-    str_result = Tcl_GetStringResult(tcl_interp);
+    // Distinguish between results set by the Tcl interpreter and results
+    // set by CDL-related commands running in that interpreter.
+    cdl_result = false;
+    
+    int result = Tcl_EvalFile(tcl_interp, CDL_TCL_CONST_CAST(char*, script.c_str()));
     // The distinction between TCL_OK and TCL_RETURN is probably not worth
     // worrying about.
     if (TCL_RETURN == result) {
         result = TCL_OK;
     }
+
+    // If we have an error condition that was raised by the Tcl
+    // interpreter rather than by the library, it needs to be
+    // raised up to the library level. That way the error count
+    // etc. are kept accurate.
+    if ((TCL_OK != result) && !cdl_result) {
+        const char* tcl_result = Tcl_GetStringResult(tcl_interp);
+        if ((0 == tcl_result) || ('\0' == tcl_result[0])) {
+            tcl_result = "Internal error, no additional information available.";
+        }
+        CdlParse::report_error(this, "", tcl_result);
+    }
+    
+    str_result = Tcl_GetStringResult(tcl_interp);
     CYG_REPORT_RETVAL(result);
     return result;
 }
 
+// Variants for when the result string is of no interest
+int
+CdlInterpreterBody::eval(std::string script)
+{
+    std::string result_string;
+    return this->eval(script, result_string);
+}
+
+int
+CdlInterpreterBody::eval_cdl_code(const cdl_tcl_code script)
+{
+    std::string result_string;
+    return this->eval_cdl_code(script, result_string);
+}
+
+int
+CdlInterpreterBody::eval_file(std::string filename)
+{
+    std::string result_string;
+    return this->eval_file(filename, result_string);
+}
+
 //}}}
 //{{{  CdlInterpreter:: set_result()                            
 
@@ -759,9 +829,22 @@ CdlInterpreterBody::set_result(std::string result)
     CYG_PRECONDITION_THISC();
 
     Tcl_SetResult(tcl_interp, const_cast<char*>(result.c_str()), TCL_VOLATILE);
+    this->cdl_result = true;
+    
     CYG_REPORT_RETURN();
 }
 
+bool
+CdlInterpreterBody::result_set_by_cdl()
+{
+    CYG_REPORT_FUNCNAMETYPE("CdlInterpreter::result_set_by_cdl", "result %d");
+    CYG_PRECONDITION_THISC();
+
+    bool result = this->cdl_result;
+    CYG_REPORT_RETVAL(result);
+    return result;
+}
+
 // ----------------------------------------------------------------------------
 // Also allow the result to be extracted again.
 std::string
@@ -785,7 +868,7 @@ CdlInterpreterBody::get_result()
 // i.e. a function pointer. That function needs a pointer to the
 // CdlInterpreter object, which can be accessed via AssocData.
 int
-CdlInterpreterBody::tcl_command_proc(ClientData data, Tcl_Interp* tcl_interp, int argc, char* argv[])
+CdlInterpreterBody::tcl_command_proc(ClientData data, Tcl_Interp* tcl_interp, int argc, const char* argv[])
 {
     CYG_REPORT_FUNCNAMETYPE("CdlInterpreter::tcl_command_proc", "result %d");
     CYG_REPORT_FUNCARG3XV(data, tcl_interp, argc);
@@ -807,14 +890,14 @@ CdlInterpreterBody::tcl_command_proc(ClientData data, Tcl_Interp* tcl_interp, in
     try {
         result = (*command)(interp, argc, argv);
     } catch(std::bad_alloc e) {
-        interp->set_result(CdlParse::get_diagnostic_prefix(interp) + "Out of memory.");
+        interp->set_result(CdlParse::construct_diagnostic(interp, "internal error", "", "Out of memory."));
         result = TCL_ERROR;
     } catch(CdlStringException e) {
         interp->set_result(e.get_message());
         result = TCL_ERROR;
     } catch(...) {
         CYG_FAIL("Unexpected C++ exception");
-        interp->set_result(CdlParse::get_diagnostic_prefix(interp) + "Internal error, unexpected C++ exception.");
+        interp->set_result(CdlParse::construct_diagnostic(interp, "internal error", "", "Unexpected C++ exception."));
         result = TCL_ERROR;
     }
 
@@ -838,9 +921,19 @@ CdlInterpreterBody::add_command(std::string name, CdlInterpreterCommand command)
     } x;
     x.command = command;
 
-    if (0 == Tcl_CreateCommand(tcl_interp, const_cast<char*>(name.c_str()), &tcl_command_proc, x.data, 0)) {
+    // Tcl 8.4 involves some incompatible API changes
+#if (TCL_MAJOR_VERSION > 8) || ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 4))
+    if (0 == Tcl_CreateCommand(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()), &tcl_command_proc, x.data, 0)) {
         throw std::bad_alloc();
     }
+#else
+    if (0 == Tcl_CreateCommand(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()),
+                               (int (*)(ClientData,Tcl_Interp*, int, char*[])) &tcl_command_proc,
+                               x.data, 0)) {
+        throw std::bad_alloc();
+    }
+#endif
+    
     CYG_REPORT_RETURN();
 }
 
@@ -856,7 +949,7 @@ CdlInterpreterBody::remove_command(std::string name)
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC("" != name);
 
-    if (0 != Tcl_DeleteCommand(tcl_interp, const_cast<char*>(name.c_str()))) {
+    if (0 != Tcl_DeleteCommand(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()))) {
         CYG_FAIL("attempt to delete non-existant command");
     }
     CYG_REPORT_RETURN();
@@ -942,7 +1035,7 @@ CdlInterpreterBody::set_variable(std::string name, std::string value)
     CYG_REPORT_FUNCARG2("this %p, name %s", this, name.c_str());
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC("" != name);
-    if (0 == Tcl_SetVar(tcl_interp, const_cast<char*>(name.c_str()), const_cast<char*>(value.c_str()), TCL_GLOBAL_ONLY)) {
+    if (0 == Tcl_SetVar(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()), CDL_TCL_CONST_CAST(char*, value.c_str()), TCL_GLOBAL_ONLY)) {
         throw std::bad_alloc();
     }
     CYG_REPORT_RETURN();
@@ -956,7 +1049,7 @@ CdlInterpreterBody::unset_variable(std::string name)
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC("" != name);
 
-    Tcl_UnsetVar(tcl_interp, const_cast<char*>(name.c_str()), TCL_GLOBAL_ONLY);
+    Tcl_UnsetVar(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()), TCL_GLOBAL_ONLY);
     CYG_REPORT_RETURN();
 }
 
@@ -969,7 +1062,7 @@ CdlInterpreterBody::get_variable(std::string name)
     CYG_PRECONDITIONC("" != name);
 
     std::string result = "";
-    char *tmp = Tcl_GetVar(tcl_interp, const_cast<char*>(name.c_str()), TCL_GLOBAL_ONLY);
+    const char *tmp = Tcl_GetVar(tcl_interp, CDL_TCL_CONST_CAST(char*, name.c_str()), TCL_GLOBAL_ONLY);
     if (0 != tmp) {
         result = tmp;
     }
@@ -996,7 +1089,7 @@ CdlInterpreterBody::set_assoc_data(const char* key, ClientData data, Tcl_InterpD
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC((0 != key) && ('\0' != key[0]));
 
-    Tcl_SetAssocData(tcl_interp, const_cast<char*>(key), del_proc, data);
+    Tcl_SetAssocData(tcl_interp, CDL_TCL_CONST_CAST(char*, key), del_proc, data);
     CYG_REPORT_RETURN();
 }
 
@@ -1008,7 +1101,7 @@ CdlInterpreterBody::get_assoc_data(const char* key)
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC((0 != key) && ('\0' != key[0]));
 
-    ClientData result = Tcl_GetAssocData(tcl_interp, const_cast<char*>(key), 0);
+    ClientData result = Tcl_GetAssocData(tcl_interp, CDL_TCL_CONST_CAST(char*, key), 0);
     CYG_REPORT_RETVAL(result);
     return result;
 }
@@ -1021,7 +1114,7 @@ CdlInterpreterBody::delete_assoc_data(const char* key)
     CYG_PRECONDITION_THISC();
     CYG_PRECONDITIONC((0 != key) && ('\0' != key[0]));
 
-    Tcl_DeleteAssocData(tcl_interp, const_cast<char*>(key));
+    Tcl_DeleteAssocData(tcl_interp, CDL_TCL_CONST_CAST(char*, key));
     CYG_REPORT_RETURN();
 }
 
@@ -1084,7 +1177,7 @@ set pattern [file join \"$::cdl_locate_subdirs_path\" * .]  \n\
 set result {}                                               \n\
 foreach entry [glob -nocomplain -- $pattern] {              \n\
     set entry [file tail [file dirname $entry]]             \n\
-    if {$entry != \"CVS\"} {                                \n\
+    if {($entry != \"CVS\") && ($entry != \"cvs\")} {       \n\
         lappend result $entry                               \n\
     }                                                       \n\
 }                                                           \n\
@@ -1097,9 +1190,9 @@ return $result                                              \n\
         CYG_FAIL("Internal error evaluating Tcl script");
     }
 
-    int         count;
-    char**      array;
-    if (TCL_OK != Tcl_SplitList(tcl_interp, const_cast<char*>(tcl_result.c_str()), &count, &array)) {
+    int             count;
+    const char**    array;
+    if (TCL_OK != Tcl_SplitList(tcl_interp, CDL_TCL_CONST_CAST(char*, tcl_result.c_str()), &count, CDL_TCL_CONST_CAST(char***, &array))) {
         throw std::bad_alloc();
     }
     for (int i = 0; i < count; i++) {
@@ -1164,9 +1257,9 @@ return $result                                          \n\
     if (TCL_OK != eval(locate_files_script, tcl_result)) {
         CYG_FAIL("Internal error evaluating Tcl script");
     }
-    int         count;
-    char**      array;
-    if (TCL_OK != Tcl_SplitList(tcl_interp, const_cast<char*>(tcl_result.c_str()), &count, &array)) {
+    int             count;
+    const char**    array;
+    if (TCL_OK != Tcl_SplitList(tcl_interp, CDL_TCL_CONST_CAST(char*, tcl_result.c_str()), &count, CDL_TCL_CONST_CAST(char***, &array))) {
         throw std::bad_alloc();
     }
     for (int i = 0; i < count; i++) {
@@ -1218,14 +1311,13 @@ CdlInterpreterBody::locate_all_files(std::string directory, std::vector<std::str
 
 void
 CdlInterpreterBody::write_data(Tcl_Channel chan, std::string data)
-    throw(CdlInputOutputException)
 {
     CYG_REPORT_FUNCNAME("CdlInterpreter::write_data");
     CYG_REPORT_FUNCARG2XV(this, chan);
     CYG_PRECONDITION_THISC();
 
-    if (-1 == Tcl_Write(chan, const_cast<char*>(data.data()), data.size())) {
-        std::string msg = "Unexpected error writing to file " + this->get_filename() + " : " + Tcl_PosixError(tcl_interp);
+    if (-1 == Tcl_Write(chan, CDL_TCL_CONST_CAST(char*, data.data()), data.size())) {
+        std::string msg = "Unexpected error writing to file " + this->get_context() + " : " + Tcl_PosixError(tcl_interp);
         throw CdlInputOutputException(msg);
     }
     
